@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,11 +20,15 @@
 
  *****************************************************************************)
 
-(** Clocks indicate at which rate a source can be read, thus avoiding by typing
-    the need for infinite buffering. *)
-
+(** A clock represents the rate at which a source runs. In itself it does not
+  * do much, except forcing that one source belongs to exactly one clock,
+  * which prevents inconsistent uses of the source. Clocks are assigned to
+  * sources at the end of the typing phase. *)
 class clock : string -> Source.clock
 
+(** Wallclocks are clocks attached to a thread that periodically makes the
+  * clock tick, which triggers the streaming of the active sources attached
+  * to the clock. *)
 class wallclock : ?sync:bool -> string -> clock
 
 class self_sync : string ->
@@ -81,4 +85,5 @@ val create_unknown : sources:(Source.active_source list) ->
                      clock_variable
 val create_known : clock -> clock_variable
 val unify : clock_variable -> clock_variable -> unit
+val forget : clock_variable -> clock_variable -> unit
 val get : clock_variable -> Source.clock
