@@ -25,11 +25,19 @@ val dynlink : bool
 (** Where to look for dynamically loadable modules (cmxs). *)
 val plugins_dir : string
 
-(** Helper to load dynamic modules. *)
-val load_dynlinks : unit -> unit
+(** Where to look for private executables. *)
+val bin_dir : string
 
-(** Helper to load dynamic plugins. *)
-val load_plugins_dir : string -> unit
+(** Standard path. *)
+val path : string list
+
+(** Helper for all things that need to be
+  * executed before script parsing. *)
+val at_init : (unit -> unit) -> unit
+val run_init : unit -> unit
+
+(** Executable extension. *)
+val exe_ext : string
 
 (** Default font file *)
 val default_font : string
@@ -46,9 +54,6 @@ val data_mime : (?len:int -> string -> string) option
 
 val requests_table_size : int
 
-(** Program used for text-to-speech. *)
-val tts_program : string
-
 (** Configured directories. Typically /var/(run|log)/liquidsoap. *)
 val rundir : string
 val logdir : string
@@ -67,3 +72,6 @@ val libs_versions : string
 
 (** File watch utility. *)
 val file_watcher : File_watcher.watch ref
+
+(** JSON parser. *)
+module JSON : (module type of JSON)
