@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
  *****************************************************************************)
 
-open Lang
 open Lang_builtins
 
 let log = Dtools.Log.make ["icy";"update_metadata"]
@@ -37,19 +36,23 @@ let () =
      "port", Lang.int_t, Some (Lang.int 8000), None;
      "user", Lang.string_t, Some (Lang.string "source"), None;
      "password", Lang.string_t, Some (Lang.string "hackme"), None;
-     "mount", Lang.string_t, None , None;
+     "mount", Lang.string_t, Some (Lang.string ""),
+       Some "Source mount point. Mandatory when streaming to icecast.";
+     "icy_id", Lang.int_t, Some (Lang.int 1),
+       Some "Shoutcast source ID. Only supported by Shoutcast v2.";
      "protocol", Lang.string_t, Some (Lang.string "http"), 
-     Some "Protocol to use. One of: \"icy\" or \"http\"";
+     Some "Protocol to use. One of: \"icy\", \"http\" or \"https\"";
      "encoding", Lang.string_t, Some (Lang.string ""),
      Some "Encoding used to send metadata, default (UTF-8) if empty." ;
      "headers", Lang.metadata_t,
-     Some (Lang.list Lang.metadata_t [user_agent]),
+     Some (Lang.list ~t:Lang.metadata_t [user_agent]),
      Some "Additional headers." ;
      "",Lang.metadata_t,None,None ] Lang.unit_t
     (fun p ->
       let user = Lang.to_string (List.assoc "user" p) in
       let password = Lang.to_string (List.assoc "password" p) in
       let mount = Lang.to_string (List.assoc "mount" p) in
+      let icy_id = Lang.to_int (List.assoc "icy_id" p) in
       let metas = Lang.to_metadata (Lang.assoc "" 1 p) in
       let out_enc =
         match Lang.to_string (List.assoc  "encoding" p) with
@@ -82,10 +85,16 @@ let () =
         let v = List.assoc "protocol" p in
         match Lang.to_string v with
           | "icy"  -> Cry.Icy
-          | "http" -> Cry.Http
+          | "http" -> Cry.Http Cry.Source (* Verb doesn't matter here. *)
+          | "https" -> Cry.Https Cry.Source
           | _      -> 
               raise (Lang.Invalid_value (v, "protocol should be one of: \
-                                             'icy' or 'http'."))
+                                             'icy', 'http' or 'https'."))
+      in
+      let mount =
+        match protocol with
+          | Cry.Icy -> Cry.Icy_id icy_id
+          | _ -> Cry.Icecast_mount mount
       in
       begin
        try
@@ -97,6 +106,6 @@ let () =
          | e ->
              log#f 2
                "Manual metadata update failed: %s"
-               (Utils.error_message e)
+               (Printexc.to_string e)
       end ;
       Lang.unit)
