@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 module Term = Lang_values
 include Term.V
 module T = Lang_types
-module Vars = Term.Vars
 
 type t = T.t
 
@@ -80,6 +79,8 @@ let univ_t ?(constraints=[]) i =
           v
 let string_getter_t n = univ_t ~constraints:[T.Getter T.String] n
 let float_getter_t n = univ_t ~constraints:[T.Getter T.Float] n
+let int_getter_t n = univ_t ~constraints:[T.Getter T.Int] n
+let bool_getter_t n = univ_t ~constraints:[T.Getter T.Bool] n
 
 let frame_kind_t ~audio ~video ~midi = Term.frame_kind_t audio video midi
 let of_frame_kind_t t = Term.of_frame_kind_t t
@@ -213,17 +214,17 @@ let kind_type_of_kind_format ~fresh fmt =
 (** Value construction *)
 
 let mk ~t v = { t = t ; value = v }
-let unit = mk unit_t Unit
-let int i = mk int_t (Int i)
-let bool i = mk bool_t (Bool i)
-let float i = mk float_t (Float i)
-let string i = mk string_t (String i)
-let product a b = mk (product_t a.t b.t) (Product (a,b))
+let unit = mk ~t:unit_t Unit
+let int i = mk ~t:int_t (Int i)
+let bool i = mk ~t:bool_t (Bool i)
+let float i = mk ~t:float_t (Float i)
+let string i = mk ~t:string_t (String i)
+let product a b = mk ~t:(product_t a.t b.t) (Product (a,b))
 
-let list ~t l = mk (list_t t) (List l)
+let list ~t l = mk ~t:(list_t t) (List l)
 
 let source s =
-  mk (source_t (kind_type_of_frame_kind s#kind)) (Source s)
+  mk ~t:(source_t (kind_type_of_frame_kind s#kind)) (Source s)
 
 let request r =
   let kind =
@@ -231,12 +232,12 @@ let request r =
       | Some k -> k
       | None -> let z = Frame.Zero in {Frame.audio=z;video=z;midi=z}
   in
-    mk (request_t (kind_type_of_frame_kind kind)) (Request r)
+    mk ~t:(request_t (kind_type_of_frame_kind kind)) (Request r)
 
 let val_fun p ~ret_t f =
   let f env t = f (List.map (fun (x,(g,v)) -> assert (g=[]) ; x,v) env) t in
   let t = fun_t (List.map (fun (l,_,t,d) -> d<>None,l,t) p) ret_t in
-  let p' = List.map (fun (l,x,t,d) -> l,x,d) p in
+  let p' = List.map (fun (l,x,_,d) -> l,x,d) p in
     mk ~t (FFI (p',[],f))
 
 let val_cst_fun p c =
@@ -257,7 +258,7 @@ let val_cst_fun p c =
 
 let metadata m =
   list
-    (product_t string_t string_t)
+    ~t:(product_t string_t string_t)
     (Hashtbl.fold
        (fun k v l -> (product (string k) (string v))::l)
        m [])
@@ -265,6 +266,23 @@ let metadata m =
 (** Runtime error, should eventually disappear. *)
 exception Invalid_value of value*string
 
+(** Helpers for defining protocols. *)
+
+let to_proto_doc ~syntax ~static doc =
+  let item = new Doc.item ~sort:false doc in
+  item#add_subsection "syntax" (Doc.trivial syntax);
+  item#add_subsection "static" (Doc.trivial (string_of_bool static));
+  item
+
+let add_protocol ~syntax ~doc ~static name resolver =
+  let doc = to_proto_doc ~syntax ~static doc in
+  let spec = {
+    Request.
+      static = static ;
+      resolve = resolver
+  } in
+  Request.protocols#register ~doc name spec
+
 (** Helpers for defining builtin functions. *)
 
 type proto = (string*t*value option*string option) list
@@ -289,7 +307,7 @@ let builtin_type p t =
   T.make
     (T.Arrow (List.map (fun (lbl,t,opt,_) -> (opt<>None,lbl,t)) p, t))
 
-let to_doc category flags main_doc proto return_t =
+let to_plugin_doc category flags main_doc proto return_t =
   let item = new Doc.item ~sort:false main_doc in
   let t = builtin_type proto return_t in
   let generalized = T.filter_vars (fun _ -> true) t in
@@ -316,7 +334,7 @@ let add_builtin ~category ~descr ?(flags=[]) name proto return_t f =
   in
   let generalized = T.filter_vars (fun _ -> true) t in
     Term.builtins#register
-      ~doc:(to_doc category flags descr proto return_t)
+      ~doc:(to_plugin_doc category flags descr proto return_t)
       name
       (generalized,value)
 
@@ -421,7 +439,7 @@ let iter_sources f v =
     | Term.Int _ | Term.Float _ | Term.Encoder _ -> ()
     | Term.List l -> List.iter (iter_term env) l
     | Term.Ref a | Term.Get a -> iter_term env a
-    | Term.Let {Term.def=a;body=b}
+    | Term.Let {Term.def=a;body=b;_}
     | Term.Product (a,b) | Term.Seq (a,b) | Term.Set (a,b) ->
         iter_term env a ; iter_term env b
     | Term.Var v ->
@@ -437,6 +455,13 @@ let iter_sources f v =
         List.iter (fun (_,_,_,v) -> match v with
                      | Some v -> iter_term env v
                      | None -> ()) proto
+    | Term.RFun (fv,proto,fn) ->
+        begin
+          match (fn()).Term.term with
+            | Term.Let {Term.body=body} ->
+                iter_term env {v with Term.term = Term.Fun (fv,proto,body)}
+            | _ -> assert false
+        end
 
   and iter_value v = match v.value with
     | Source s -> f s
@@ -473,12 +498,12 @@ let iter_sources f v =
          * which probably won't prevent users to get biffled... *)
         let may_have_source =
           try
-            let has_var_neg,has_var_pos =
+            let _,has_var_pos =
               Lang_types.iter_constr
                 (fun pos c ->
                    if pos &&
                       match c with
-                        | { T.name = "source" } -> true | _ -> false
+                      | { T.name = "source"; _} -> true | _ -> false
                    then raise Found)
                 v.t
             in
@@ -509,6 +534,15 @@ let to_bool t = match t.value with
   | Bool b -> b
   | _ -> assert false
 
+let to_bool_getter t = match t.value with
+  | Bool b -> (fun () -> b)
+  | Fun _ | FFI _ ->
+     (fun () ->
+       match (apply ~t:bool_t t []).value with
+       | Bool b -> b
+       | _ -> assert false)
+  | _ -> assert false
+
 let to_fun ~t f =
   match f.value with
   | Fun _  | FFI _ ->
@@ -557,6 +591,15 @@ let to_int t = match t.value with
   | Int s -> s
   | _ -> assert false
 
+let to_int_getter t = match t.value with
+  | Int n -> (fun () -> n)
+  | Fun _ | FFI _ ->
+     (fun () ->
+       match (apply ~t:int_t t []).value with
+       | Int n -> n
+       | _ -> assert false)
+  | _ -> assert false
+
 let to_list t = match t.value with
   | List l -> l
   | _ -> assert false
@@ -565,12 +608,15 @@ let to_product t = match t.value with
   | Product (a,b) -> (a,b)
   | _ -> assert false
 
-let to_metadata t = 
-  let pop v = 
+let to_metadata_list t =
+  let pop v =
     let f (a,b) = (to_string a,to_string b) in
     f (to_product v)
   in
-  let t = List.map pop (to_list t) in
+  List.map pop (to_list t)
+
+let to_metadata t = 
+  let t = to_metadata_list t in
   let metas = Hashtbl.create 10 in
   List.iter (fun (a,b) -> Hashtbl.add metas a b) t;
   metas
@@ -628,7 +674,7 @@ let report_error lexbuf f =
           " before %S: %s!\n" (Lexing.lexeme lexbuf) error
   in
     try f () with
-      | Failure "lexing: empty token" -> print_error "Empty token" ; raise Error
+      | Failure s when s = "lexing: empty token" -> print_error "Empty token" ; raise Error
       | Parsing.Parse_error -> print_error "Parse error" ; raise Error
       | Lang_values.Parse_error (pos,s) ->
         let pos = T.print_pos pos in
@@ -837,7 +883,7 @@ let interactive () =
         | Error ->
             true
         | e ->
-            Format.printf "Exception: %s!@." (Utils.error_message e) ;
+            Format.printf "Exception: %s!@." (Printexc.to_string e) ;
             true
     then
       loop ()
