@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,6 +46,9 @@ and in_value =
   | Ref     of value ref
   | Fun     of (string * string * value option) list *
                full_env * full_env * Lang_values.term
+  (** A function with given arguments (argument label, argument variable,
+      default value), parameters already passed to the function, closure and
+      value. *)
   | FFI     of (string * string * value option) list *
                full_env *
                (full_env -> t -> value)
@@ -65,6 +68,12 @@ val iter_sources : (Source.source -> unit) -> value -> unit
 (** Multiapply a value to arguments. *)
 val apply : value -> env -> t:t -> value
 
+(** {3 Helpers for registering protocols} *)
+
+val add_protocol :
+  syntax:string -> doc:string ->
+  static:bool -> string -> Request.resolver -> unit
+
 (** {3 Helpers for source builtins} *)
 
 type proto =
@@ -155,6 +164,7 @@ val add_operator :
 
 val to_unit : value -> unit
 val to_bool : value -> bool
+val to_bool_getter : value -> unit -> bool
 val to_string : value -> string
 val to_string_getter : value -> unit -> string
 val to_float : value -> float
@@ -163,8 +173,10 @@ val to_source : value -> Source.source
 val to_format : value -> Encoder.format
 val to_request : value -> Request.t
 val to_int : value -> int
+val to_int_getter : value -> unit -> int
 val to_list : value -> value list
 val to_product : value -> value * value
+val to_metadata_list : value -> (string*string) list
 val to_metadata : value -> Frame.metadata
 val to_string_list : value -> string list
 val to_int_list : value -> int list
@@ -225,6 +237,8 @@ val string_getter_t : int -> t
 (** A float getter. The argument is the number of the universal type parameter
   * (should be >= 1). *)
 val float_getter_t : int -> t
+val int_getter_t : int -> t
+val bool_getter_t : int -> t
 
 val unit : value
 val int : int -> value
