@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,42 +23,34 @@
 (* TODO: also parse optional arguments? *)
 let get_encoder_format tokenizer lexbuf =
   let ogg_item = function
-    | Lang_parser.VORBIS -> Lang_encoders.mk_vorbis []
-    | Lang_parser.VORBIS_CBR -> Lang_encoders.mk_vorbis_cbr []
-    | Lang_parser.VORBIS_ABR -> Lang_encoders.mk_vorbis_abr []
-    | Lang_parser.THEORA -> Lang_encoders.mk_theora []
-    | Lang_parser.DIRAC -> Lang_encoders.mk_dirac []
-    | Lang_parser.SPEEX -> Lang_encoders.mk_speex []
-    | Lang_parser.OPUS -> Lang_encoders.mk_opus []
-    | Lang_parser.FLAC -> Lang_encoders.mk_ogg_flac []
+    | Lang_parser.VORBIS -> Lang_vorbis.make []
+    | Lang_parser.VORBIS_CBR -> Lang_vorbis.make_cbr []
+    | Lang_parser.VORBIS_ABR -> Lang_vorbis.make_abr []
+    | Lang_parser.THEORA -> Lang_theora.make []
+    | Lang_parser.SPEEX -> Lang_speex.make []
+    | Lang_parser.OPUS -> Lang_opus.make []
+    | Lang_parser.FLAC -> Lang_flac.make_ogg []
     | _ -> failwith "ogg format expected"
   in
   let is_ogg_item token =
     try let _ = ogg_item token in true with _ -> false
   in
-  let fmt =
     match tokenizer lexbuf with
-    | Lang_parser.MP3 -> Lang_encoders.mk_mp3_cbr []
-    | Lang_parser.MP3_VBR -> Lang_encoders.mk_mp3_vbr []
-    | Lang_parser.MP3_ABR -> Lang_encoders.mk_mp3_vbr []
-    | Lang_parser.MP3_FXP -> Lang_encoders.mk_shine []
-    | Lang_parser.AACPLUS -> Lang_encoders.mk_aacplus []
-    | Lang_parser.VOAACENC -> Lang_encoders.mk_voaacenc []
-    | Lang_parser.FDKAAC -> Lang_encoders.mk_fdkaac []
-    | Lang_parser.FLAC -> Lang_encoders.mk_flac []
-    | Lang_parser.EXTERNAL -> Lang_encoders.mk_external []
-    | Lang_parser.GSTREAMER -> Lang_encoders.mk_gstreamer []
-    | Lang_parser.WAV -> Lang_encoders.mk_wav []
+    | Lang_parser.MP3 -> Lang_mp3.make_cbr []
+    | Lang_parser.MP3_VBR -> Lang_mp3.make_vbr []
+    | Lang_parser.MP3_ABR -> Lang_mp3.make_vbr []
+    | Lang_parser.SHINE   -> Lang_shine.make []
+    | Lang_parser.FDKAAC -> Lang_fdkaac.make []
+    | Lang_parser.FLAC -> Lang_flac.make []
+    | Lang_parser.EXTERNAL -> Lang_external_encoder.make []
+    | Lang_parser.GSTREAMER -> Lang_gstreamer.make []
+    | Lang_parser.WAV -> Lang_wav.make []
     | ogg when is_ogg_item ogg ->
       let ogg = ogg_item ogg in
-      Lang_encoders.mk (Lang_values.Encoder (Encoder.Ogg [ogg]))
+        Encoder.Ogg [ogg]
     (* TODO *)
     (* | Lang_parser.OGG -> Lang_encoders.mk ... [] *)
-    | _ -> failwith "encoder format expected"
-  in
-  match fmt.Lang_values.term with
-  | Lang_values.Encoder fmt -> fmt
-  | _ -> assert false
+    | _ -> failwith "expected an encoding format after %ifencoder"
 
 (* The Lang_lexer is not quite enough for our needs,
  * so we first define convenient layers between it and the parser.
@@ -76,20 +68,28 @@ let preprocess tokenizer =
       | _ -> skip ()
     in
     match tokenizer lexbuf with
-      | Lang_parser.PP_IFDEF ->
+      | Lang_parser.PP_IFDEF | Lang_parser.PP_IFNDEF as tok ->
           begin match tokenizer lexbuf with
             | Lang_parser.VAR v ->
+                let test =
+                  if tok = Lang_parser.PP_IFDEF then fun x -> x else not
+                in
                 (** XXX Less natural meaning than the original one. *)
-                if Lang_values.builtins#is_registered v then
+                if test (Lang_values.builtins#is_registered v) then
                   go_on ()
                 else
                   skip ()
             | _ -> failwith "expected a variable after %ifdef"
           end
-      | Lang_parser.PP_IFENCODER ->
-        let fmt = get_encoder_format tokenizer lexbuf in
-        let has_enc = try let _ = Encoder.get_factory fmt in true with Not_found -> false in
-        if has_enc then go_on () else skip ()
+      | Lang_parser.PP_IFENCODER | Lang_parser.PP_IFNENCODER as tok ->
+          let fmt = get_encoder_format tokenizer lexbuf in
+          let has_enc =
+            try let _ = Encoder.get_factory fmt in true with Not_found -> false
+          in
+          let test =
+            if tok = Lang_parser.PP_IFENCODER then fun x -> x else not
+          in
+            if test has_enc then go_on () else skip ()
       | Lang_parser.PP_ENDIF ->
           if !state=0 then failwith "no %ifdef to end here" ;
           decr state ;
@@ -262,7 +262,9 @@ let expand tokenizer =
 let parse_comments tokenizer =
   let documented_def doc =
     let doc =
-      List.map (Pcre.substitute ~pat:"^\\s*#\\s?" ~subst:(fun _ -> "")) doc
+      List.map
+        (fun x -> Pcre.substitute ~pat:"^\\s*#\\s?" ~subst:(fun _ -> "") x)
+        doc
     in
     let rec parse_doc (main,special,params) = function
       | [] -> (main,special,params)
@@ -386,7 +388,7 @@ let expand_define tokenizer =
       (
         match tokenizer lexbuf with
         | Lang_parser.VAR x ->
-          if x <> String.uppercase x then raise Parsing.Parse_error;
+          if x <> Utils.StringCompat.uppercase_ascii x then raise Parsing.Parse_error;
           (
             match tokenizer lexbuf with
             | Lang_parser.INT _
