@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@ let debug =
   with
     | Not_found -> false
 
-(** Should errors be considered as simple warnings? *)
-let errors_as_warnings = ref false
+(** Should some warnings be considered as fatal errors? *)
+let strict = ref false
 
 (** {2 Kinds} *)
 
@@ -43,8 +43,8 @@ let errors_as_warnings = ref false
 
 module T = Lang_types
 
-let ref_t ~pos ~level t =
-  T.make ~pos ~level
+let ref_t ?pos ?level t =
+  T.make ?pos ?level
     (T.Constr { T.name = "ref" ; T.params = [T.Invariant,t] })
 
 let zero_t = T.make T.Zero
@@ -66,7 +66,7 @@ let frame_kind_t ?pos ?level audio video midi =
 let of_frame_kind_t t = match (T.deref t).T.descr with
   | T.Constr { T.name="stream_kind" ; T.params=[_,audio;_,video;_,midi] } ->
       { Frame. audio=audio; video=video; midi=midi }
-  | T.EVar (j,c) ->
+  | T.EVar (_,_) ->
       let audio = type_of_int (Lazy.force Frame.audio_channels) in
       let video = type_of_int (Lazy.force Frame.video_channels) in
       let midi = type_of_int (Lazy.force Frame.midi_channels) in
@@ -148,6 +148,9 @@ and in_term =
 | Var     of string
 | Seq     of term * term
 | App     of term * (string * term) list
+| RFun    of Vars.t *
+    (string*string*T.t*term option) list *
+    (unit -> term)
 | Fun     of Vars.t *
     (string*string*T.t*term option) list *
     term
@@ -180,7 +183,7 @@ let rec print_term v = match v.term with
   | Ref a ->
       Printf.sprintf "ref(%s)" (print_term a)
   | Fun (_,[],v) when is_ground v -> "{"^(print_term v)^"}"
-  | Fun _ -> "<fun>"
+  | Fun _ | RFun _ -> "<fun>"
   | Var s -> s
   | App (hd,tl) ->
       let tl =
@@ -205,8 +208,8 @@ let rec free_vars tm = match tm.term with
         (fun v (_,t) -> Vars.union v (free_vars t))
         (free_vars hd)
         l
-  | Fun (fv,_,_) ->
-      fv
+  | RFun (fv,_,_)
+  | Fun (fv,_,_) -> fv
   | Let l ->
       Vars.union
         (free_vars l.def)
@@ -220,7 +223,7 @@ let free_vars ?bound body =
 
 let can_ignore t =
   match (T.deref t).T.descr with
-    | T.Ground T.Unit | T.Constr {T.name="active_source"} -> true
+    | T.Ground T.Unit | T.Constr {T.name="active_source";_} -> true
     | T.EVar _ -> true
     | _ -> false
 
@@ -230,7 +233,7 @@ let is_fun t =
 
 let is_source t =
   match (T.deref t).T.descr with
-    | T.Constr {T.name="source"} -> true | _ -> false
+    | T.Constr {T.name="source";_} -> true | _ -> false
 
 (** Check that all let-bound variables are used.
   * No check is performed for variable arguments.
@@ -240,7 +243,7 @@ let is_source t =
 
 exception Unused_variable of (string*Lexing.position)
 
-let rec check_unused ~lib tm =
+let check_unused ~lib tm =
   let rec check ?(toplevel=false) v tm = match tm.term with
     | Var s -> Vars.remove s v
     | Unit | Bool _ | Int _ | String _ | Float _ | Encoder _ -> v
@@ -248,11 +251,22 @@ let rec check_unused ~lib tm =
     | Get r -> check v r
     | Product (a,b) | Set (a,b) -> check (check v a) b
     | Seq (a,b) -> check ~toplevel (check v a) b
-    | List l -> List.fold_left check v l
+    | List l -> List.fold_left (fun x y -> check x y) v l
     | App (hd,l) ->
         let v = check v hd in
-          List.fold_left (fun v (lbl,t) -> check v t) v l
-    | Fun (fv,p,body) ->
+          List.fold_left (fun v (_,t) -> check v t) v l
+    (* A recursive function may not use its recursive let. *)
+    | RFun (fv,p,fn) ->
+      begin
+        match (fn()).term with
+          | Let {var=var;body=body} ->
+              let v =
+                check v {tm with term = Fun (fv,p,body)}
+              in
+              Vars.remove var v        
+          | _ -> assert false
+      end
+    | Fun (_,p,body) ->
         let v =
           List.fold_left
             (fun v -> function
@@ -275,7 +289,7 @@ let rec check_unused ~lib tm =
            * The masking variables have been used but it does not count
            * for the ones they masked. *)
           Vars.union masked v
-    | Let { var = s ; def = def ; body = body } ->
+    | Let { var = s ; def = def ; body = body ; _ } ->
         let v = check v def in
         let mask = Vars.mem s v in
         let v = Vars.add s v in
@@ -289,12 +303,12 @@ let rec check_unused ~lib tm =
              * and functions when at toplevel (sort of a lib situation...) *)
             if not (can_ignore def.t || (toplevel && is_fun def.t)) then
               let start_pos = fst (Utils.get_some tm.t.T.pos) in
-                if !errors_as_warnings then
+                if !strict then
+                  raise (Unused_variable (s,start_pos))
+                else
                   Printf.printf
                     "Warning: unused variable %s at %s.\n%!"
                     s (T.print_single_pos start_pos)
-                else
-                  raise (Unused_variable (s,start_pos))
           end ;
           if mask then Vars.add s v else v
   in
@@ -303,7 +317,12 @@ let rec check_unused ~lib tm =
 
 (** Maps a function on all types occurring in a term.
   * Ignores variable generalizations. *)
-let rec map_types f (gen:'a list) tm = match tm.term with
+let rec map_types f (gen:'a list) tm =
+  let aux = function
+    | (lbl,var,t,None) -> lbl, var, f gen t, None
+    | (lbl,var,t,Some tm) -> lbl, var, f gen t, Some (map_types f gen tm)
+  in
+  match tm.term with
   | Unit | Bool _ | Int _ | String _ | Float _ | Encoder _ | Var _ ->
       { tm with t = f gen tm.t }
   | Ref r ->
@@ -329,12 +348,14 @@ let rec map_types f (gen:'a list) tm = match tm.term with
         term = App (map_types f gen hd,
                     List.map (fun (lbl,v) -> lbl, map_types f gen v) l) }
   | Fun (fv,p,v) ->
-      let aux = function
-        | (lbl,var,t,None) -> lbl, var, f gen t, None
-        | (lbl,var,t,Some tm) -> lbl, var, f gen t, Some (map_types f gen tm)
+      { t = f gen tm.t ;
+        term = Fun (fv, List.map aux p, map_types f gen v) }
+  | RFun (fv,p,fn) ->
+      let fn () =
+        map_types f gen (fn ())
       in
-        { t = f gen tm.t ;
-          term = Fun (fv, List.map aux p, map_types f gen v) }
+      { t = f gen tm.t ;
+        term = RFun (fv, List.map aux p, fn) }
   | Let l ->
       let gen' = l.gen@gen in
         { t = f gen tm.t ;
@@ -344,7 +365,17 @@ let rec map_types f (gen:'a list) tm = match tm.term with
 (** Folds [f] over almost all types occurring in a term,
   * skipping as much as possible while still
   * guaranteeing that [f] will see all variables. *)
-let rec fold_types f gen x tm = match tm.term with
+let rec fold_types f gen x tm =
+  let fold_proto x p =
+    List.fold_left
+      (fun x -> function
+        | (_,_,t,Some tm) ->
+            fold_types f gen (f gen x t) tm
+        | (_,_,t,None) ->
+            f gen x t)
+     x p
+  in
+  match tm.term with
   | Unit | Bool _ | Int _ | String _ | Float _ | Encoder _ | Var _ ->
       f gen x tm.t
   | List l ->
@@ -358,17 +389,16 @@ let rec fold_types f gen x tm = match tm.term with
       let x = fold_types f gen x tm in
         List.fold_left (fun x (_,tm) -> fold_types f gen x tm) x l
   | Fun (_,p,v) ->
-      fold_types f gen
-        (List.fold_left
-           (fun x -> function
-              | (_,_,t,Some tm) ->
-                  fold_types f gen (f gen x t) tm
-              | (_,_,t,None) ->
-                  f gen x t)
-           x
-           p)
-        v
-  | Let {gen=gen';def=def;body=body} ->
+      fold_types f gen (fold_proto x p) v
+  | RFun (_,p,fn) ->
+      begin
+        match (fn()).term with
+          | Let l ->
+              let x = f gen x l.def.t in
+              fold_types f gen (fold_proto x p) l.body
+          | _ -> assert false
+      end 
+  | Let {gen=gen';def=def;body=body;_} ->
       let x = fold_types f (gen'@gen) x def in
         fold_types f gen x body
 
@@ -401,14 +431,21 @@ struct
 
   type env = (string*value) list
 
+  let string_of_float f =
+    let s = string_of_float f in
+    if s.[String.length s -1] = '.' then
+      (s ^ "0")
+    else
+      s
+
   let rec print_value v = match v.value with
     | Unit     -> "()"
     | Bool i   -> string_of_bool i
     | Int i    -> string_of_int i
     | Float f  -> string_of_float f
     | String s -> Printf.sprintf "%S" s
-    | Source s -> "<source>"
-    | Request s -> "<request>"
+    | Source _ -> "<source>"
+    | Request _ -> "<request>"
     | Encoder e -> Encoder.string_of_format e
     | List l ->
         "["^(String.concat ", " (List.map print_value l))^"]"
@@ -469,10 +506,10 @@ struct
      * no type instantiation should occur in the following cases (f should
      * be the identity): one cannot change the type of such objects once
      * they are created. *)
-    | Source s ->
+    | Source _ ->
         assert (f gen v.t = v.t) ;
         v
-    | Request s ->
+    | Request _ ->
         assert (f gen v.t = v.t) ;
         v
     | Ref r ->
@@ -503,12 +540,12 @@ exception Unbound of T.pos option * string
 exception Ignored of term
 
 let raise_ignored e =
-  if !errors_as_warnings then
+  if !strict then
+    raise (Ignored e)
+  else
     Printf.printf
       "Warning: ignored expression at %s.\n%!"
       (T.print_pos ~prefix:"" (Utils.get_some e.t.T.pos))
-  else
-    raise (Ignored e)
 
 (** [No_label (f,lbl,first,x)] indicates that the parameter [x] could not be
   * passed to the function [f] because the latter has no label [lbl].
@@ -542,6 +579,32 @@ let rec check ?(print_toplevel=false) ~level ~env e =
   let pos = e.t.T.pos in
   let mk t = T.make ~level ~pos t in
   let mkg t = mk (T.Ground t) in
+  let check_fun ~proto ~env e body =
+    let base_check = check ~level ~env in
+    let proto_t,env,level =
+      List.fold_left
+        (fun (p,env,level) -> function
+           | lbl,var,kind,None   ->
+               if debug then
+                 Printf.eprintf "Assigning level %d to %s (%s).\n"
+                   level var (T.print kind) ;
+               kind.T.level <- level ;
+               (false,lbl,kind)::p, (var,([],kind))::env, level+1
+           | lbl,var,kind,Some v ->
+               if debug then
+                 Printf.eprintf "Assigning level %d to %s (%s).\n"
+                   level var (T.print kind) ;
+               kind.T.level <- level ;
+               base_check v ;
+               v.t <: kind ;
+               (true,lbl,kind)::p, (var,([],kind))::env, level+1)
+        ([],env,level)
+        proto
+    in
+    let proto_t = List.rev proto_t in
+      check ~level ~env body ;
+      e.t >: mk (T.Arrow (proto_t,body.t))
+  in
   match e.term with
   | Unit      -> e.t >: mkg T.Unit
   | Bool    _ -> e.t >: mkg T.Bool
@@ -550,19 +613,27 @@ let rec check ?(print_toplevel=false) ~level ~env e =
   | Float   _ -> e.t >: mkg T.Float
   | Encoder f -> e.t >: type_of_format ~pos:e.t.T.pos ~level f
   | List l ->
-      List.iter (check ~level ~env) l ;
-      let pos =
-        (* Attach the position of the first item in the list
-         * to the type of the list items. It gives more info with type errors
-         * when the items in the list are not compatible.
-         * WARNING This will become weird with real subtyping in the inference,
-         * because the type of elements will only be _a supertype_ of the type
-         * of the first item. *)
-        match l with e::_ -> e.t.T.pos | [] -> e.t.T.pos
+      List.iter (fun x -> check ~level ~env x) l ;
+      (* We first try to compute the sup of types of elements in the list,
+         which will give us the type of the list. *)
+      let tsup =
+        List.fold_left
+          (fun sup e ->
+            try
+              e.t >: sup;
+              e.t
+            with
+              | T.Type_Error _ ->
+                  if debug then
+                    Printf.eprintf "Ignoring type error to compute \
+                                    a sup of list element types.\n" ;
+                  e.t <: sup;
+                  sup)
+          (T.fresh_evar ~level ~pos)
+          l
       in
-      let v = T.fresh_evar ~level ~pos in
-        e.t >: mk (T.List v) ;
-        List.iter (fun item -> item.t <: v) l
+        e.t >: mk (T.List tsup) ;
+        List.iter (fun item -> item.t <: tsup) l
   | Product (a,b) ->
       check ~level ~env a ; check ~level ~env b ;
       e.t >: mk (T.Product (a.t,b.t))
@@ -593,7 +664,7 @@ let rec check ?(print_toplevel=false) ~level ~env e =
         | T.Arrow (ap,t) ->
             (* Find in l the first arg labeled lbl,
              * return it together with the remaining of the list. *)
-            let rec get_arg lbl l =
+            let get_arg lbl l =
               let rec aux acc = function
                 | [] -> None
                 | (o,lbl',t)::l ->
@@ -613,7 +684,7 @@ let rec check ?(print_toplevel=false) ~level ~env e =
                      | None ->
                          let first = not (List.mem lbl already) in
                            raise (No_label (a,lbl,first,v))
-                     | Some (o,t,ap') ->
+                     | Some (_,t,ap') ->
                          v.t <: t ;
                          (lbl::already,ap'))
                 ([],ap)
@@ -629,31 +700,16 @@ let rec check ?(print_toplevel=false) ~level ~env e =
             let p = List.map (fun (lbl,b) -> false,lbl,b.t) l in
               a.t <: T.make ~level ~pos:None (T.Arrow (p,e.t))
       end
-  | Fun (_,proto,body) ->
-      let base_check = check ~level ~env in
-      let proto_t,env,level =
-        List.fold_left
-          (fun (p,env,level) -> function
-             | lbl,var,kind,None   ->
-                 if debug then
-                   Printf.eprintf "Assigning level %d to %s (%s).\n"
-                     level var (T.print kind) ;
-                 kind.T.level <- level ;
-                 (false,lbl,kind)::p, (var,([],kind))::env, level+1
-             | lbl,var,kind,Some v ->
-                 if debug then
-                   Printf.eprintf "Assigning level %d to %s (%s).\n"
-                     level var (T.print kind) ;
-                 kind.T.level <- level ;
-                 base_check v ;
-                 v.t <: kind ;
-                 (true,lbl,kind)::p, (var,([],kind))::env, level+1)
-          ([],env,level)
-          proto
-      in
-      let proto_t = List.rev proto_t in
-        check ~level ~env body ;
-        e.t >: mk (T.Arrow (proto_t,body.t))
+  | Fun (_,proto,body) -> check_fun ~proto ~env e body
+  | RFun (_,proto,fn) -> 
+      begin
+        match (fn()).term with
+          | Let {var=var;def=def;body=body} ->
+             let env = (var,([],def.t))::env in
+             check_fun ~proto ~env def body;
+             e.t >: def.t
+          | _ -> assert false
+      end 
   | Var var ->
       let generalized,orig =
         try
@@ -669,7 +725,7 @@ let rec check ?(print_toplevel=false) ~level ~env e =
         if debug then
           Printf.eprintf "Instantiate %s[%d] : %s becomes %s\n"
             var (T.deref e.t).T.level (T.print orig) (T.print e.t)
-  | Let ({gen=gen; var=name; def=def; body=body} as l) ->
+  | Let ({var=name; def=def; body=body; _} as l) ->
       check ~level:level ~env def ;
       let generalized =
         if value_restriction def then
@@ -677,7 +733,7 @@ let rec check ?(print_toplevel=false) ~level ~env e =
             let x' =
               T.filter_vars
                 (function
-                   | { T. descr = T.EVar (i,c) ; level = l } ->
+                   | { T. descr = T.EVar (i,_) ; level = l ; _ } ->
                        not (List.mem_assoc i x) &&
                        not (List.mem_assoc i gen) && l >= level
                    | _ -> assert false)
@@ -695,7 +751,7 @@ let rec check ?(print_toplevel=false) ~level ~env e =
           (add_task (fun () ->
              Format.printf "@[<2>%s :@ %a@]@."
                (let l = String.length name and max = 5 in
-                  if l >= max then name else name ^ String.make (max-l) ' ')
+                  if l >= max then name else name ^ Bytes.make (max-l) ' ')
                (T.pp_type_generalized generalized) def.t)) ;
         check ~print_toplevel ~level:(level+1) ~env body ;
         e.t >: body.t
@@ -793,6 +849,21 @@ let lookup env var ty =
     v
 
 let rec eval ~env tm =
+  let prepare_fun fv p env =
+    (* Unlike OCaml we always evaluate default values,
+     * and we do that early.
+     * I think the only reason is homogeneity with FFI,
+     * which are declared with values as defaults. *)
+    let p =
+      List.map
+        (function
+          | (lbl,var,_,Some v) -> lbl,var,Some (eval ~env v)
+          | (lbl,var,_,None) -> lbl,var,None)
+      p
+    in
+    let env = List.filter (fun (x,_) -> Vars.mem x fv) env in
+    (p,env)
+  in
   let mk v = { V.t = tm.t ; V.value = v } in
     match tm.term with
       | Unit -> mk (V.Unit)
@@ -816,26 +887,30 @@ let rec eval ~env tm =
                 mk V.Unit
             | _ -> assert false
           end
-      | Let {gen=generalized;var=x;def=v;body=b} ->
+      | Let {gen=generalized;var=x;def=v;body=b;_} ->
           (* It should be the case that generalizable variables don't
            * get instantiated in any way when evaluating the definition.
            * But we don't double-check it. *)
           let v = eval ~env v in
           eval ~env:((x,(generalized,v))::env) b
       | Fun (fv,p,body) ->
-          (* Unlike OCaml we always evaluate default values,
-           * and we do that early.
-           * I think the only reason is homogeneity with FFI,
-           * which are declared with values as defaults. *)
-          let p =
-            List.map
-              (function
-                 | (lbl,var,_,Some v) -> lbl,var,Some (eval ~env v)
-                 | (lbl,var,_,None) -> lbl,var,None)
-              p
-          in
-          let env = List.filter (fun (x,_) -> Vars.mem x fv) env in
+          let (p,env) = prepare_fun fv p env in
             mk (V.Fun (p,[],env,body))
+      | RFun (fv,p,fn) ->
+          begin
+            match (fn ()).term with
+              | Let {var=var;body=body} ->
+                  let (p,env) = prepare_fun fv p env in
+                  let rec ffi args t =
+                    let v = mk (V.FFI (p,[],ffi)) in
+                    let env = (var,([],v))::env in
+                    let env = List.rev_append args env in
+                    let f = mk (V.Fun ([],[],env,body)) in
+                    apply ~t f []
+                  in
+                    mk (V.FFI (p,[],ffi))
+              | _ -> assert false
+          end
       | Var var ->
           lookup env var tm.t
       | Seq (a,b) ->
@@ -914,9 +989,9 @@ let toplevel_add (doc,params) x ~generalized v =
       | V.Fun (p,_,_,_) -> List.map (fun (l,_,o) -> l,o) p
       | _ -> []
   in
-  let params,pvalues =
+  let params,_ =
     List.fold_left
-      (fun (params,pvalues) (opt,label,t) ->
+      (fun (params,pvalues) (_,label,t) ->
          let descr,params =
            try
              List.assoc label params,
