@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  *****************************************************************************)
 
 open Dtools
-open Printf
 
 (** Runner module signature. *)
 module type Runner_t =
@@ -268,6 +267,7 @@ struct
 
   let descr_key t p =
     try
+      load_libs () ;
       print_string (descr ~prefix:(Conf.path_of_string p) t);
       exit 0
     with
@@ -284,14 +284,17 @@ struct
       "Describe a configuration key.";
       ["--conf-descr"],
       Arg.Unit (fun () ->
+        load_libs () ;
         print_string (descr t); exit 0),
       "Display a described table of the configuration keys.";
       ["--conf-descr-liqi"],
       Arg.Unit (fun () ->
+        load_libs () ;
         print_string (descr ~liqi:true t); exit 0),
       "Display a described table of the configuration keys in liqi (documentation wiki) format.";
       ["--conf-dump"],
       Arg.Unit (fun () ->
+        load_libs () ;
         print_string (dump t); exit 0),
       "Dump the configuration state";
     ]
@@ -382,16 +385,16 @@ let options = [
       else
         [])
     @
-    [["--errors-as-warnings"],
-    Arg.Set Lang_values.errors_as_warnings,
-    "Issue warnings instead of fatal errors for unused variables \
-     and ignored expressions. If you are not sure about it, it is better \
-     to not use it." ]
+    [["--strict"],
+    Arg.Set Lang_values.strict,
+    "Execute script code in strict mode, issuing fatal errors \
+     instead of warnings in some cases. Currently: unused variables \
+     and ignored expressions. " ]
     @
     (* Unix.fork is not implemented in Win32. *)
     (if Sys.os_type <> "Win32" then
       [["-d";"--daemon"],
-       Arg.Unit (fun f -> Init.conf_daemon#set true),
+       Arg.Unit (fun _ -> Init.conf_daemon#set true),
        "Run in daemon mode."]
      else [])
     @
@@ -444,7 +447,7 @@ let options = [
     Arg.Unit (fun () ->
                 Printf.printf
                   "Liquidsoap %s%s\n\
-                   Copyright (c) 2003-2013 Savonet team\n\
+                   Copyright (c) 2003-2017 Savonet team\n\
                    Liquidsoap is open-source software, \
                    released under GNU General Public License.\n\
                    See <http://liquidsoap.fm> for more information.\n"
@@ -519,15 +522,11 @@ struct
     else
       s
 
-  (* Load plugins and dynamic loaded libraries:
-   * these should be loaded as early as possible since we want to be
-   * able to use them in scripts... *)
   let () =
-    Configure.load_dynlinks ();
-    Configure.load_plugins_dir Configure.plugins_dir
+    Configure.run_init ()
 
   (* Startup *)
-  let () =
+  let startup () =
     Random.self_init () ;
 
     (* Set the default values. *)
@@ -572,15 +571,27 @@ struct
 
   (* Now that outputs have been defined, we can start the main loop. *)
   let () =
-    let cleanup () =
+    let cleanup_threads () =
       log#f 3 "Shutdown started!" ;
       Clock.stop () ;
       log#f 3 "Waiting for threads to terminate..." ;
-      Tutils.join_all () ;
+      Tutils.join_all ();
+      log#f 3 "Threads terminated."
+    in
+    let cleanup_final () =
       log#f 3 "Cleaning downloaded files..." ;
       Request.clean () ;
       log#f 3 "Freeing memory..." ;
-      Gc.full_major ()
+      Gc.full_major ();
+      if !Shutdown.restart then
+        (
+          log#f 3 "Restarting..." ;
+          Unix.execv Sys.executable_name Sys.argv
+        )
+    in
+    let cleanup () =
+      cleanup_threads ();
+      cleanup_final ()
     in
     let main () =
       (* See http://caml.inria.fr/mantis/print_bug_page.php?bug_id=4640
@@ -602,7 +613,10 @@ struct
       Clock.start () ;
       Tutils.main ()
     in
-      ignore (Init.at_stop cleanup) ;
+      (* We join threads, then shutdown duppy, then do the final task. *)
+      ignore (Init.at_stop ~before:[Shutdown.duppy_scheduler ()] cleanup_threads);
+      Shutdown.final_atom := Some (Init.at_stop ~depends:[Shutdown.duppy_scheduler ()] cleanup_final);
+      startup ();
       if !interactive then begin
         load_libs () ;
         Log.conf_stdout#set_d (Some false) ;
