@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@ open Source
 
 class echo ~kind (source:source) delay feedback ping_pong =
   let channels = (Frame.type_of_kind kind).Frame.audio in
-object (self)
-  inherit operator ~name:"echo" kind [source] as super
+object
+  inherit operator ~name:"echo" kind [source]
 
   method stype = source#stype
   method remaining = source#remaining
