@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,39 +21,35 @@
  *****************************************************************************)
 
 class on_end ~kind ~delay f s =
-object (self)
+object(self)
   inherit Source.operator ~name:"on_end" kind [s]
+  inherit Latest_metadata.source
 
   val mutable executed = false
 
-  val mutable latest_metadata = None
-
   method stype = s#stype
   method is_ready = s#is_ready
   method remaining = s#remaining
   method abort_track = s#abort_track
   method seek n = s#seek n
 
+  method private on_new_metadata = ()
+
   method private get_frame ab =
     s#get ab ;
-    let compare x y = - (compare x y) in
-    let l = List.sort compare (Frame.get_all_metadata ab) in
-    if List.length l > 0 then
-      latest_metadata <- Some (snd (List.hd l));
+    self#save_latest_metadata ab ;
     let rem = Frame.seconds_of_master s#remaining in
     if (not executed) && ((0. <= rem && rem <= delay) || Frame.is_partial ab) then
     begin
-      let m =
-        match latest_metadata with
-          | Some m -> m
-          | None -> Hashtbl.create 0
-      in
       ignore(Lang.apply ~t:Lang.unit_t f ["",Lang.float rem;
-                                          "",Lang.metadata m]) ;
+                                          "",Lang.metadata latest_metadata]) ;
       executed <- true
     end ;
     if Frame.is_partial ab then
-      executed <- false ;
+     begin
+      self#clear_latest_metadata ;
+      executed <- false
+     end
 end
 
 let () =
@@ -65,12 +61,12 @@ let () =
             equal to this value." ;
       "",
       Lang.fun_t
-        [(false,"",Lang.float_t);
-          false,"",Lang.list_t (Lang.product_t Lang.string_t Lang.string_t)]
+        [(false,"",Lang.float_t);(false,"",Lang.metadata_t)]
         Lang.unit_t,
       None,
       Some "Function to execute. First argument is the remaining time, \
-            second is the latest metadata." ;
+            second is the latest metadata. That function should be fast \
+            because it is executed in the main streaming thread." ;
       "", Lang.source_t kind, None, None ]
     ~category:Lang.TrackProcessing
     ~descr:"Call a given handler when there is less than \
