@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ object (self)
   val mutable gen_before = Generator.create ()
   val mutable rms_before = 0.
   val mutable rmsi_before = 0
-  val mutable mem_before = Array.create rms_width 0.
+  val mutable mem_before = Array.make rms_width 0.
   val mutable mem_i = 0
   val mutable before_metadata = None
 
@@ -76,7 +76,7 @@ object (self)
    * pull data from it at a higher rate around track limits. *)
   val mutable source = s
 
-  method private wake_up activator =
+  method private wake_up _ =
     s#get_ready ~dynamic:true [(self:>source)] ;
     source <- s ;
     source#get_ready [(self:>source)] ;
@@ -109,7 +109,9 @@ object (self)
     Clock.unify slave_clock s#clock ;
     Lang.iter_sources
       (fun s -> Clock.unify slave_clock s#clock)
-      transition
+      transition ;
+    (* Make sure the slave clock can be garbage collected, cf. cue_cut(). *)
+    Gc.finalise (fun self -> Clock.forget self#clock slave_clock) self
 
   val mutable master_time = 0
   val mutable last_slave_tick = 0 (* in master time *)
@@ -223,9 +225,10 @@ object (self)
         rmsi_before <- min rms_width (rmsi_before + 1)
     done ;
     (* Should we buffer more or are we done ? *)
-    if AFrame.is_partial buf_frame then
+    if AFrame.is_partial buf_frame then begin
+      Generator.add_break gen_before ;
       status <- `Limit
-    else
+    end else
       if n>0 then self#buffering (n - AFrame.position buf_frame)
 
   (* Analyze the beginning of a new track. *)
@@ -292,7 +295,7 @@ object (self)
            in
            let metadata = function
              | None ->
-                 Lang.list (Lang.product_t Lang.string_t Lang.string_t) []
+                 Lang.list ~t:(Lang.product_t Lang.string_t Lang.string_t) []
              | Some m -> Lang.metadata m
            in
            let f a b =
@@ -374,7 +377,7 @@ let () =
       "width", Lang.float_t, Some (Lang.float 1.),
       Some "Width of the power computation window." ;
 
-      "conservative", Lang.bool_t, Some (Lang.bool false),
+      "conservative", Lang.bool_t, Some (Lang.bool true),
       Some "Do not trust remaining time estimations, always buffering \
             data in advance. This avoids being tricked by skips, either \
             manual or caused by skip_blank()." ;
