@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@ open Source
 
 class bpm ~kind (source:source) cb every =
   let every = Frame.audio_of_seconds every in
-object (self)
-  inherit operator ~name:"bpm" kind [source] as super
+object
+  inherit operator ~name:"bpm" kind [source]
 
   method stype = source#stype
   method is_ready = source#is_ready
