@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
   * A few specializations of it are defined below. *)
 
 open Source
-open Dtools
 
 (* A transition is a value of type (source,source) -> source *)
 type transition = Lang.value
@@ -46,10 +45,9 @@ type child = {
 type track_mode = Sensitive | Insensitive
 
 class virtual switch ~kind ~name
-  ?(mode=Sensitive) ?(replay_meta=true) (cases : child list) =
+  ?(mode=(fun () -> true)) ?(replay_meta=true) (cases : child list) =
 object (self)
-
-  inherit operator ~name kind (List.map (fun x -> x.source) cases) as super
+  inherit operator ~name kind (List.map (fun x -> x.source) cases)
 
   val mutable selected : (child*source) option = None
 
@@ -68,15 +66,16 @@ object (self)
 
   (** Don't call #select directly but use #cached_select
     * to ensure consistency during one time tick between #is_ready and
-    * #get_frame. *)
+    * #get_frame. We want to make sure that when a source is selected
+    * during a #is_ready call, the same selection is returned during the 
+    * next #get_frame call. *)
   val mutable cached_selected = None
   method private cached_select =
     match cached_selected with
-      | Some c -> c
+      | Some _ as c -> c
       | None ->
-          let c = self#select in
-            cached_selected <- Some c ;
-            c
+          cached_selected <- self#select ;
+          cached_selected
 
   method after_output =
     (* Advance the memo frame. *)
@@ -102,7 +101,7 @@ object (self)
   method private wake_up activator =
     activation <- (self:>source)::activator ;
     List.iter
-      (fun { transition = transition ; source = s } ->
+      (fun { transition = transition ; source = s ; _ } ->
          s#get_ready ~dynamic:true activation ;
          Lang.iter_sources
            (fun s -> s#get_ready ~dynamic:true activation)
@@ -111,7 +110,7 @@ object (self)
 
   method private sleep =
     List.iter
-      (fun { transition = transition ; source = s } ->
+      (fun { transition = transition ; source = s ; _ } ->
          s#leave ~dynamic:true (self:>source) ;
          Lang.iter_sources (fun s -> s#leave ~dynamic:true (self:>source))
            transition)
@@ -181,7 +180,7 @@ object (self)
           end
       | None ->
           begin match selected with
-            | Some (old_c,old_s) ->
+            | Some (_,old_s) ->
                 old_s#leave (self:>source) ;
                 to_finish <- old_s::to_finish ;
                 selected <- None
@@ -219,11 +218,11 @@ object (self)
                    (Frame.get_all_metadata ab)
                with
                  | None -> None
-                 | Some (_,m) -> Some m) ;
+                 | Some (_,m) -> Some (Hashtbl.copy m)) ;
           if Frame.is_partial ab then
             reselect ~forget:true ()
           else
-            if mode = Insensitive then
+            if not (mode ()) then
               reselect ()
 
   method remaining =
@@ -241,7 +240,7 @@ end
 (** Common tools for Lang bindings of switch operators *)
 
 let common kind = [
-  "track_sensitive", Lang.bool_t, Some (Lang.bool true),
+  "track_sensitive", Lang.bool_getter_t 0, Some (Lang.bool true),
   Some "Re-select only on end of tracks." ;
 
   "replay_metadata", Lang.bool_t, Some (Lang.bool true),
@@ -256,7 +255,7 @@ let common kind = [
   in
   "transitions",
   Lang.list_t transition_t,
-  Some (Lang.list transition_t []),
+  Some (Lang.list ~t:transition_t []),
   Some "Transition functions, \
         padded with <code>fun (x,y) -> y</code> functions."
 ]
@@ -268,12 +267,7 @@ let default_transition k =
       (fun e _ -> List.assoc "y" e)
 
 let extract_common ~kind p l =
-  let ts =
-    if Lang.to_bool (List.assoc "track_sensitive" p) then
-      Sensitive
-    else
-      Insensitive
-  in
+  let ts = Lang.to_bool_getter (List.assoc "track_sensitive" p) in
   let tr =
     let tr = Lang.to_list (List.assoc "transitions" p) in
     let ltr = List.length tr in
@@ -293,15 +287,14 @@ let extract_common ~kind p l =
 let satisfied f = Lang.to_bool (Lang.apply ~t:Lang.bool_t f [])
 let trivially_true = function
   | { Lang.value =
-        Lang.Fun (_,_,_,{ Lang_values.term = Lang_values.Bool true }) } -> true
+        Lang.Fun (_,_,_,{ Lang_values.term = Lang_values.Bool true; _}); _} -> true
   | _ -> false
 
 let third (_,_,s) = s
 
 class lang_switch ~kind
   mode ?replay_meta (children : (Lang.value * bool * child) list) =
-object (self)
-
+object
   inherit
     switch ~name:"switch" ~kind ~mode ?replay_meta (List.map third children)
 
@@ -338,7 +331,7 @@ let () =
   let kind = Lang.univ_t 1 in
   let pred_t = Lang.fun_t [] Lang.bool_t in
   let proto =
-    [ "single", Lang.list_t Lang.bool_t, Some (Lang.list Lang.bool_t []),
+    [ "single", Lang.list_t Lang.bool_t, Some (Lang.list ~t:Lang.bool_t []),
       Some "Forbid the selection of a branch for two tracks in a row. \
             The empty list stands for <code>[false,...,false]</code>." ;
       "", Lang.list_t (Lang.product_t pred_t (Lang.source_t kind)), None,
@@ -378,7 +371,7 @@ let () =
          in
          let children =
            try List.map2 (fun (d,s) single -> d,single,s) children singles with
-             | Invalid_argument("List.map2") ->
+             | Invalid_argument s when s = "List.map2" ->
                  raise (Lang.Invalid_value
                           (List.assoc "single" p,
                            "there should be exactly one flag per children"))
@@ -430,7 +423,7 @@ let () =
 (** Random switch *)
 exception Found of child
 class random ~kind ?replay_meta strict mode children =
-  let name = if strict then "quota" else "random" in
+  let name = if strict then "rotate" else "random" in
 object
   inherit switch ~name ~kind ?replay_meta ~mode (List.map snd children)
 
@@ -469,7 +462,7 @@ let () =
     let kind = Lang.univ_t 1 in
     Lang.add_operator name ~descr ~category:Lang.TrackProcessing
       (common  kind @
-       [ "weights", Lang.list_t Lang.int_t, Some (Lang.list Lang.int_t []),
+       [ "weights", Lang.list_t Lang.int_t, Some (Lang.list ~t:Lang.int_t []),
          Some weight_descr ;
          "", Lang.list_t (Lang.source_t kind), None, None ])
       ~kind:(Lang.Unconstrained kind)
