@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 module Img = Image.RGBA32
 module GU = Gstreamer_utils
 
-let init () = GU.init ()
+let init () = ()
 
 let render_text ~font ~size text =
   (* Override default value... *)
