@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ let get_font font size =
     | Sdlttf.SDLttf_exception s ->
       raise (Lang.Invalid_value (Lang.string font, s))
     | e ->
-      raise (Lang.Invalid_value (Lang.string font, Utils.error_message e))
+      raise (Lang.Invalid_value (Lang.string font, Printexc.to_string e))
 
 let render_text ~font ~size text =
   let text = if text = "" then " " else text in
