@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ class output ~kind ~clock_safe ~nb_blocks ~driver
   let samples_per_second = Lazy.force Frame.audio_rate in
   let bytes_per_sample = 2 in
   let blank () =
-    String.make (samples_per_frame * channels * bytes_per_sample) '0'
+    Bytes.make (samples_per_frame * channels * bytes_per_sample) '0'
   in
 object (self)
   inherit Output.output  ~content_kind:kind
@@ -133,7 +133,7 @@ let () =
 
       "options",
       Lang.metadata_t,
-      Some (Lang.list (Lang.product_t Lang.string_t Lang.string_t) []),
+      Some (Lang.list ~t:(Lang.product_t Lang.string_t Lang.string_t) []),
       Some "List of parameters, depends on the driver." ;
 
       "", Lang.source_t kind, None, None
