@@ -1,6 +1,6 @@
 (*****************************************************************************
 
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 class output ~kind ~infallible ~autostart ~on_start ~on_stop source =
   let video_width    = Lazy.force Frame.video_width in
   let video_height   = Lazy.force Frame.video_height in
-object (self)
+object
   inherit Output.output ~name:"graphics" ~output_kind:"output.graphics"
             ~infallible ~on_start ~on_stop ~content_kind:kind source autostart
 
