@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
 
 (** Output to an icecast server. *)
 
-open Dtools
-
 let error_translator =
   function
     | Cry.Error _ as e ->
@@ -42,14 +40,6 @@ type icecast_info =
 
 module Icecast = 
 struct
-
-  type protocol = Cry.protocol
-
-  let protocol_of_icecast_protocol = 
-    function
-      | Icecast_utils.Http -> Cry.Http
-      | Icecast_utils.Icy -> Cry.Icy 
-
   type content = Cry.content_type
 
   let format_of_content x =
@@ -66,88 +56,82 @@ struct
     function
         | Encoder.MP3 m ->
             let quality,bitrate = 
-              match m.Encoder.MP3.bitrate_control with
-                | Encoder.MP3.CBR x -> None,(Some x)
-                | Encoder.MP3.ABR x -> None,(Some x.Encoder.MP3.mean_bitrate)
-                | Encoder.MP3.VBR q -> (Some (string_of_int q)),None
+              match m.Mp3_format.bitrate_control with
+                | Mp3_format.CBR x -> None,(Some x)
+                | Mp3_format.ABR x -> None,(Some x.Mp3_format.mean_bitrate)
+                | Mp3_format.VBR q -> (Some (string_of_int q)),None
             in
             { quality = quality ;
               bitrate = bitrate ;
-              samplerate = Some m.Encoder.MP3.samplerate ;
-              channels = Some (if m.Encoder.MP3.stereo then 2 else 1)
+              samplerate = Some m.Mp3_format.samplerate ;
+              channels = Some (if m.Mp3_format.stereo then 2 else 1)
             }
         | Encoder.Shine m ->
             { quality = None ;
-              bitrate = Some m.Encoder.Shine.bitrate ;
-              samplerate = Some m.Encoder.Shine.samplerate ;
-              channels = Some m.Encoder.Shine.channels
-            }
-        | Encoder.AACPlus m ->
-            { quality = None ;
-              bitrate = Some m.Encoder.AACPlus.bitrate ;
-              samplerate = Some m.Encoder.AACPlus.samplerate ;
-              channels = Some m.Encoder.AACPlus.channels
-            }
-        | Encoder.VoAacEnc m ->
-            { quality = None ;
-              bitrate = Some m.Encoder.VoAacEnc.bitrate ;
-              samplerate = Some m.Encoder.VoAacEnc.samplerate ;
-              channels = Some m.Encoder.VoAacEnc.channels
+              bitrate = Some m.Shine_format.bitrate ;
+              samplerate = Some m.Shine_format.samplerate ;
+              channels = Some m.Shine_format.channels
             }
         | Encoder.FdkAacEnc m ->
             { quality = None ;
-              bitrate = Some m.Encoder.FdkAacEnc.bitrate ;
-              samplerate = Some m.Encoder.FdkAacEnc.samplerate ;
-              channels = Some m.Encoder.FdkAacEnc.channels
+              bitrate = Some m.Fdkaac_format.bitrate ;
+              samplerate = Some m.Fdkaac_format.samplerate ;
+              channels = Some m.Fdkaac_format.channels
             }
         | Encoder.External m ->
             { quality = None ;
               bitrate = None ;
-              samplerate = Some m.Encoder.External.samplerate ;
-              channels = Some m.Encoder.External.channels
+              samplerate = Some m.External_encoder_format.samplerate ;
+              channels = Some m.External_encoder_format.channels
             }
         | Encoder.GStreamer m ->
             { quality    = None ;
               bitrate    = None ;
               samplerate = None ;
-              channels   = Some (Encoder.GStreamer.audio_channels m)
+              channels   = Some (Gstreamer_format.audio_channels m)
             }
         | Encoder.Flac m ->
-            { quality = Some (string_of_int m.Encoder.Flac.compression) ;
+            { quality = Some (string_of_int m.Flac_format.compression) ;
               bitrate = None ;
-              samplerate = Some m.Encoder.Flac.samplerate ;
-              channels = Some m.Encoder.Flac.channels
+              samplerate = Some m.Flac_format.samplerate ;
+              channels = Some m.Flac_format.channels
             }
         | Encoder.WAV m ->
             { quality = None ;
               bitrate = None ;
-              samplerate = Some m.Encoder.WAV.samplerate ;
-              channels = Some m.Encoder.WAV.channels
+              samplerate = Some m.Wav_format.samplerate ;
+              channels = Some m.Wav_format.channels
+            }
+        | Encoder.AVI m ->
+            { quality = None ;
+              bitrate = None ;
+              samplerate = Some m.Avi_format.samplerate ;
+              channels = Some m.Avi_format.channels
             }
         | Encoder.Ogg o ->
             match o with
-              | [Encoder.Ogg.Vorbis
-                   {Encoder.Vorbis.channels=n;
-                                   mode=Encoder.Vorbis.VBR q;
-                                   samplerate=s}]
+              | [Ogg_format.Vorbis
+                   {Vorbis_format.channels=n;
+                                   mode=Vorbis_format.VBR q;
+                                   samplerate=s;_}]
                 ->
                   { quality = Some (string_of_float q) ;
                     bitrate = None ;
                     samplerate = Some s ;
                     channels = Some n }
-              | [Encoder.Ogg.Vorbis
-                   {Encoder.Vorbis.channels=n;
-                                   mode=Encoder.Vorbis.ABR (_,b,_);
-                                   samplerate=s}]
+              | [Ogg_format.Vorbis
+                   {Vorbis_format.channels=n;
+                                   mode=Vorbis_format.ABR (_,b,_);
+                                   samplerate=s;_}]
                 ->
                   { quality = None ;
                     bitrate = b ;
                     samplerate = Some s ;
                     channels = Some n }
-              | [Encoder.Ogg.Vorbis
-                   {Encoder.Vorbis.channels=n;
-                                   mode=Encoder.Vorbis.CBR b;
-                                   samplerate=s}]
+              | [Ogg_format.Vorbis
+                   {Vorbis_format.channels=n;
+                                   mode=Vorbis_format.CBR b;
+                                   samplerate=s;_}]
                 ->
                   { quality = None ;
                     bitrate = Some b ;
@@ -162,7 +146,7 @@ module M = Icecast_utils.Icecast_v(Icecast)
 
 open M
 
-let no_mount = "Use [name] with .ogg extension if relevant"
+let no_mount = "Use [name]"
 let no_name = "Use [mount]"
 
 let user_agent = Lang.product (Lang.string "User-Agent")
@@ -170,7 +154,10 @@ let user_agent = Lang.product (Lang.string "User-Agent")
 
 let proto kind =
   Output.proto @ (Icecast_utils.base_proto kind) @
-  [ "mount", Lang.string_t, Some (Lang.string no_mount), None ;
+  [ "mount", Lang.string_t, Some (Lang.string no_mount),
+     Some "Source mount point. Mandatory when streaming to icecast." ;
+    "icy_id", Lang.int_t, Some (Lang.int 1),
+     Some "Shoutcast source ID. Only supported by Shoutcast v2." ;
     "name", Lang.string_t, Some (Lang.string no_name), None ;
     "host", Lang.string_t, Some (Lang.string "localhost"), None ;
     "port", Lang.int_t, Some (Lang.int 8000), None ;
@@ -178,18 +165,29 @@ let proto kind =
     Some "Timeout for establishing network connections (disabled is negative).";
     "timeout", Lang.float_t, Some (Lang.float 30.),
     Some "Timeout for network read and write.";
-    ("user", Lang.string_t, Some (Lang.string "source"),
-     Some "User for shout source connection. \
+    ("user", Lang.string_t, Some (Lang.string ""),
+     Some "User for shout source connection. Defaults to \"source\" for icecast connections. \
            Useful only in special cases, like with per-mountpoint users.") ;
     "password", Lang.string_t, Some (Lang.string "hackme"), None ;
     "encoding", Lang.string_t, Some (Lang.string ""),
      Some "Encoding used to send metadata. If empty, defaults to \"UTF-8\" \
-           for \"http\" protocol and \"ISO-8859-1\" for \"icy\" \
+           for \"http(s)\" protocol and \"ISO-8859-1\" for \"icy\" \
            protocol." ;
-    "genre", Lang.string_t, Some (Lang.string "Misc"), None ;
-    "url", Lang.string_t, Some (Lang.string "http://savonet.sf.net"), None ;
-    ("description", Lang.string_t,
-     Some (Lang.string "Liquidsoap Radio!"), None) ;
+    "genre", Lang.string_t, Some (Lang.string ""), None ;
+    "protocol", Lang.string_t, (Some (Lang.string "http")),
+     Some "Protocol of the streaming server: \
+          'http' or 'https' for Icecast, 'icy' for shoutcast." ;
+    "verb", Lang.string_t, (Some (Lang.string "source")),
+     Some "Verb to use with the 'http(s)' protocol. One of: \
+           'source', 'put' or 'post'.";
+    "chunked", Lang.bool_t, (Some (Lang.bool false)),
+     Some "Used cunked transfer with the 'http(s)' protocol.";
+    "icy_metadata", Lang.string_t, Some (Lang.string "guess"),
+     Some "Send new metadata using the ICY protocol. \
+           One of: \"guess\", \"true\", \"false\"";
+    "url", Lang.string_t, Some (Lang.string ""), None ;
+    "description", Lang.string_t,
+     Some (Lang.string ""), None ;
     "on_connect",
     Lang.fun_t [] Lang.unit_t,
     Some (Lang.val_cst_fun [] Lang.unit),
@@ -207,7 +205,7 @@ let proto kind =
           this amount of time (in seconds)." ;
     "public", Lang.bool_t, Some (Lang.bool true), None ;
     ("headers", Lang.metadata_t,
-     Some (Lang.list (Lang.product_t Lang.string_t Lang.string_t) [user_agent]),
+     Some (Lang.list ~t:(Lang.product_t Lang.string_t Lang.string_t) [user_agent]),
      Some "Additional headers.") ;
     ("dumpfile", Lang.string_t, Some (Lang.string ""), 
      Some "Dump stream to file, for debugging purpose. Disabled if empty.") ;
@@ -226,14 +224,62 @@ class output ~kind p =
   let on_connect () = ignore (Lang.apply ~t:Lang.unit_t on_connect []) in
   let on_disconnect () = ignore (Lang.apply ~t:Lang.unit_t on_disconnect []) in
   let on_error error =
-    let msg = Utils.error_message error in
+    let msg = Printexc.to_string error in
     Lang.to_float (Lang.apply ~t:Lang.unit_t on_error ["", Lang.string msg]) 
   in
 
-  let protocol,encoder_factory, 
-      format,icecast_info,
-      icy_metadata,ogg =
-    encoder_data p
+  let data = encoder_data p in
+
+  let chunked = Lang.to_bool (List.assoc "chunked" p) in
+
+  let protocol = 
+    let verb =
+      let v = List.assoc "verb" p in
+      match Lang.to_string v with
+        | "source" -> Cry.Source
+        | "put"    -> Cry.Put
+        | "post"   -> Cry.Post
+        | _ -> raise (Lang.Invalid_value
+                   (v, "Valid values are: 'source' \
+                       'put' or 'post'.")) 
+    in  
+    let v = List.assoc "protocol" p in
+    match Lang.to_string v with
+      | "http" -> Cry.Http verb
+      | "https" -> Cry.Https verb
+      | "icy" -> Cry.Icy
+      | _ ->
+          raise (Lang.Invalid_value
+                   (v, "Valid values are 'http' (icecast) \
+                        and 'icy' (shoutcast)"))
+  in
+  let icy_metadata =
+    let v = List.assoc "icy_metadata" p in
+    let icy =
+       match Lang.to_string v with
+         | "guess" -> `Guess
+         | "true"  -> `True
+         | "false" -> `False
+         | _ ->
+               raise (Lang.Invalid_value
+                       (v, "Valid values are 'guess', \
+                           'true' or 'false'"))
+    in
+    match data.format, icy with
+      | _, `True -> true
+      | _, `False -> false
+      | x, `Guess when x = mpeg ||
+                       x = wav ||
+                       x = aac ||
+                       x = flac -> true
+      | x, `Guess when x = ogg_application ||
+                       x = ogg_audio ||
+                       x = ogg_video -> false
+      | _, _ ->
+           raise (Lang.Invalid_value
+                    (List.assoc "icy_metadata" p,
+                     "Could not guess icy_metadata for this format, \
+                     please specify either 'true' or 'false'."))
   in
 
   let out_enc =
@@ -243,29 +289,32 @@ class output ~kind p =
            "ISO-8859-1"
          else
            "UTF-8"
-      | s -> String.uppercase s
+      | s -> Utils.StringCompat.uppercase_ascii s
   in
 
   let source = Lang.assoc "" 2 p in
 
+  let icy_id =
+    Lang.to_int (List.assoc "icy_id" p)
+  in
+
   let mount = s "mount" in
+
   let name = s "name" in
-  let name =
-    if name = no_name then
-      if mount = no_mount then
+  let mount, name =
+    match protocol, name, mount with
+      | Cry.Http _, name, mount when name = no_name && mount = no_mount ->
         raise (Lang.Invalid_value
                  (List.assoc "mount" p,
-                  "Either name or mount must be defined"))
-      else
-        mount
-    else
-      name
-  in
-  let mount =
-    if mount = no_mount then
-      if ogg then name ^ ".ogg" else name
-    else
-      mount
+                  "Either name or mount must be defined for icecast sources."))
+      | Cry.Icy, name, _ when name = no_name ->
+         (Cry.Icy_id icy_id), (Printf.sprintf "sc#%i" icy_id)
+      | Cry.Icy, name, _ ->
+         (Cry.Icy_id icy_id), name
+      | _, name, mount when name = no_name ->
+         (Cry.Icecast_mount mount), mount
+      | _ ->
+         (Cry.Icecast_mount mount), name
   in
 
   let autostart = Lang.to_bool (List.assoc "start" p) in
@@ -281,7 +330,12 @@ class output ~kind p =
 
   let host = s "host" in
   let port = e Lang.to_int "port" in
-  let user = s "user" in
+  let user =
+    match protocol, s "user" with
+      | Cry.Http _, ""
+      | Cry.Https _, "" -> "source"
+      | _, user -> user 
+  in
   let password = s "password" in
   let genre = s "genre" in
   let url = s "url" in
@@ -315,7 +369,7 @@ object (self)
   inherit Output.encoded
             ~content_kind:kind ~output_kind:"output.icecast"
             ~infallible ~autostart ~on_start ~on_stop
-            ~name:mount source
+            ~name source
 
   (** In this operator, we don't exactly follow the start/stop
     * mechanism of Output.encoded because we want to control
@@ -349,7 +403,7 @@ object (self)
     (* Update metadata using ICY if told to.. *)
     if icy_metadata then
      begin
-      let get h k l =
+      let get h l k =
         try
           (k,(Hashtbl.find h k))::l
         with _ -> l
@@ -359,9 +413,9 @@ object (self)
           (k,(Hashtbl.find h k))::l
         with _ -> (k,d)::l
       in
-      let m = Encoder.Meta.to_metadata m in
+      let m = Meta_format.to_metadata m in
       let def_title =
-        match get m "uri" [] with
+        match get m [] "uri" with
           | (_,s)::_ -> let title = Filename.basename s in
               ( try
                   String.sub title 0 (String.rindex title '.')
@@ -374,15 +428,11 @@ object (self)
         (try Hashtbl.find m "title" with _ -> "Unknown")
       in
       let a = Array.of_list
-        (getd m "title" def_title
-           (get m "artist"
-              (get m "genre"
-                 (get m "date"
-                    (get m "album"
-                       (get m "tracknum"
-                          (get m "comment"
-                             (* for Shoutcast *)
-                             (getd m "song" default_song []))))))))
+       (getd m "title" def_title
+         (getd m "song" default_song
+           (List.fold_left (get m) []
+             ["artist";"genre";"date";"album";
+              "tracknum";"comment";"dj";"next"])))
       in
       let f = Configure.recode_tag ~out_enc in
       let a = Array.map (fun (x,y) -> (x, f y)) a in
@@ -397,7 +447,7 @@ object (self)
                  Cry.update_metadata ~charset:out_enc connection m 
                with e -> self#log#f 3 "Metadata update may have failed with \
                                        error: %s" 
-                             (Utils.error_message e))
+                             (Printexc.to_string e))
           | Cry.Disconnected -> ()
               (* Do nothing if shout connection isn't available *)
      end 
@@ -421,7 +471,7 @@ object (self)
               | None -> () 
           with
             | e ->
-                self#log#f 2 "Error while sending data: %s!" (Utils.error_message e) ;
+                self#log#f 2 "Error while sending data: %s!" (Printexc.to_string e) ;
                 let delay = on_error e in
                 if delay >= 0. then
                  begin
@@ -450,25 +500,30 @@ object (self)
 
   method icecast_start =
     assert (encoder = None) ;
-    let enc = encoder_factory self#id in
+    let enc = data.factory self#id in
     encoder <- 
-       Some (enc (Encoder.Meta.empty_metadata)) ; 
+       Some (enc (Meta_format.empty_metadata)) ; 
     assert (Cry.get_status connection = Cry.Disconnected) ;
     begin match dumpfile with
       | Some f -> dump <- Some (open_out_bin f)
       | None -> ()
     end ;
-    self#log#f 3 "Connecting mount %s for %s@%s..." mount user host ;
+    let display_mount =
+      match mount with
+        | Cry.Icy_id id -> Printf.sprintf "sid#%d" id
+        | Cry.Icecast_mount mount -> mount
+    in
+    self#log#f 3 "Connecting mount %s for %s@%s..." display_mount user host ;
     let audio_info = Hashtbl.create 10 in
     let f x y z =
       match x with
         | Some q -> Hashtbl.add audio_info y (z q)
         | None -> ()
     in
-      f icecast_info.bitrate "bitrate" string_of_int;
-      f icecast_info.quality "quality" (fun x -> x);
-      f icecast_info.samplerate "samplerate" string_of_int;
-      f icecast_info.channels "channels" string_of_int;
+      f data.info.bitrate "bitrate" string_of_int;
+      f data.info.quality "quality" (fun x -> x);
+      f data.info.samplerate "samplerate" string_of_int;
+      f data.info.channels "channels" string_of_int;
       let user_agent =
         try
           List.assoc "User-Agent" headers
@@ -478,26 +533,24 @@ object (self)
       let source = 
         Cry.connection ~host ~port ~user ~password
                        ~genre ~url ~description ~name
-                       ~public ~protocol ~mount  
-                       ~audio_info ~user_agent ~content_type:format ()
+                       ~public ~protocol ~mount ~chunked 
+                       ~audio_info ~user_agent ~content_type:data.format ()
       in
       List.iter (fun (x,y) -> 
                       (* User-Agent has already been passed to Cry.. *)
                       if x <> "User-Agent" then 
-                        Hashtbl.add source.Cry.headers x y) headers;
+                        Hashtbl.replace source.Cry.headers x y) headers;
 
       try
         Cry.connect connection source ;
         self#log#f 3 "Connection setup was successful." ;
-        let c = Cry.get_connection_data connection in
-        Unix.setsockopt c.Cry.data_socket Unix.TCP_NODELAY true;
         (* Execute on_connect hook. *)
         on_connect () ;
       with
         (* In restart mode, no_connect and no_login are not fatal.
          * The output will just try to reconnect later. *)
         | e ->
-            self#log#f 2 "Connection failed: %s" (Utils.error_message e) ;
+            self#log#f 2 "Connection failed: %s" (Printexc.to_string e) ;
             let delay = on_error e in
             if delay >= 0. then
              begin
@@ -537,4 +590,4 @@ let () =
     ~descr:"Encode and output the stream to an icecast2 or shoutcast server."
     (proto k)
     ~kind:(Lang.Unconstrained k)
-    (fun p kind -> ((new output kind p):>Source.source))
+    (fun p kind -> ((new output ~kind p):>Source.source))
