@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -165,11 +165,16 @@ object (self)
 
     if is_started then begin
       (* Complete filling of the frame *)
+      let get_count = ref 0 in
       while Frame.is_partial memo && self#is_ready do
+        incr get_count ;
+        if !get_count > Lazy.force Frame.size then
+          self#log#f 2
+            "Warning: there may be an infinite sequence of empty tracks!" ;
         source#get memo
       done ;
       List.iter
-        (fun (i,m) -> self#add_metadata m)
+        (fun (_,m) -> self#add_metadata m)
         (Frame.get_all_metadata memo) ;
       (* Output that frame *)
       self#output_send memo ;
@@ -235,7 +240,7 @@ object (self)
             ~infallible ~on_start ~on_stop
             ~content_kind ~output_kind ~name source autostart
 
-  method virtual private insert_metadata : Encoder.Meta.export_metadata -> unit
+  method virtual private insert_metadata : Meta_format.export_metadata -> unit
   method virtual private encode : Frame.t -> int -> int -> string
   method virtual private send : string -> unit
 
@@ -245,7 +250,7 @@ object (self)
         begin
           match Frame.get_metadata frame start with
             | None -> ()
-            | Some m -> self#insert_metadata (Encoder.Meta.export_metadata m)
+            | Some m -> self#insert_metadata (Meta_format.export_metadata m)
         end ;
         let data =
           self#encode frame start (stop-start)
