@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -104,7 +104,7 @@ object (self)
       match current_metadata with
         | Some m -> 
             fun x -> 
-             subst (Hashtbl.find (Encoder.Meta.to_metadata m) x)
+             subst (Hashtbl.find (Meta_format.to_metadata m) x)
         | None -> fun _ -> raise Not_found
     in
     Utils.interpolate current_metadata s
@@ -119,7 +119,7 @@ object (self)
     let meta = 
       match current_metadata with
         | Some m -> m
-        | None -> Encoder.Meta.empty_metadata
+        | None -> Meta_format.empty_metadata
     in
     encoder <- Some (enc meta) ;
 
@@ -304,7 +304,9 @@ object (self)
     Unix.open_process_out process
 
   method close_chan chan =
-    ignore(Unix.close_process_out chan)
+    try
+      ignore(Unix.close_process_out chan)
+    with Sys_error msg when msg = "Broken pipe" -> ()
 end
 
 let () =
