@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 open Genlex
 exception Error
 
-let annotate s ~log maxtime =
+let annotate s ~log _ =
   try
     (* Avoid =- being lexed as a single identifier. *)
     let s = Pcre.substitute ~pat:"=-" ~subst:(fun _ -> "= -") s in
@@ -67,7 +67,6 @@ let annotate s ~log maxtime =
     | Stream.Failure | Stream.Error _ -> log "annotate: syntax error" ; []
 
 let () =
-  Request.protocols#register "annotate"
-    ~sdoc:("[annotate:key=\"val\",key2=\"val2\",...:uri] adds "^
-           "the metadata to the request and is then resolved into uri")
-    { Request.resolve = annotate ; Request.static = false }
+  Lang.add_protocol ~doc:"Add metadata to a request"
+    ~syntax:"annotate:key=\"val\",key2=\"val2\",...:uri"
+    ~static:false "annotate" annotate
