@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
 
  *****************************************************************************)
 
-let debug = false
-
 module Conf = Dtools.Conf
 
 let conf =
@@ -40,10 +38,9 @@ let conf_path_prefix =
 let conf_randomize =
   Conf.bool ~p:(conf#plug "randomize") ~d:true
     "Randomize order of MPD's results."
-
-let re_newline = Str.regexp "[\r\n]+"
-let re_version = Str.regexp "OK MPD \\([0-9a-z\\.]+\\)"
-let re_file = Str.regexp "^file: \\(.*\\)$"
+let conf_debug =
+  Conf.bool ~p:(conf#plug "debug") ~d:false
+    "Debug communications with MPD server."
 
 exception Error of string
 
@@ -52,40 +49,40 @@ let connect () =
     try
       Unix.gethostbyname conf_host#get
     with
-      | Not_found -> raise (Error "Host not found")
+    | Not_found -> raise (Error "Host not found")
   in
   let sockaddr = Unix.ADDR_INET(host.Unix.h_addr_list.(0), conf_port#get) in
   let socket = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
   let read () =
     let buflen = 1024 in
-    let buf = String.create buflen in
+    let buf = Bytes.create buflen in
     let ans = ref "" in
     let n = ref buflen in
-      while !n = buflen do
-        n := Unix.recv socket buf 0 buflen [];
-        ans := !ans ^ String.sub buf 0 !n
-      done;
-      if debug then Printf.printf "R: %s%!" !ans;
-      !ans
+    while !n = buflen do
+      n := Unix.recv socket buf 0 buflen [];
+      ans := !ans ^ String.sub buf 0 !n
+    done;
+    if conf_debug#get then Printf.printf "R: %s%!" !ans;
+    !ans
   in
   let write s =
     let len = String.length s in
-      if debug then Printf.printf "W: %s%!" s;
-      let l = Unix.send socket s 0 len [] in
-      assert (l = len)
+    if conf_debug#get then Printf.printf "W: %s%!" s;
+    let l = Unix.send socket s 0 len [] in
+    assert (l = len)
   in
-    Unix.connect socket sockaddr;
-    socket, read, write
+  Unix.connect socket sockaddr;
+  socket, read, write
 
 let re_newline = Str.regexp "[\r\n]+"
 
 let cmd read write name args =
   let args = List.map (fun s -> "\"" ^ s ^ "\"") args in
-    write (name ^ " " ^ String.concat " " args ^ "\n");
-    let ans = read () in
-    let ans = Str.split re_newline ans in
-    let ans = List.rev ans in
-      List.hd ans, List.rev (List.tl ans)
+  write (name ^ " " ^ String.concat " " args ^ "\n");
+  let ans = read () in
+  let ans = Str.split re_newline ans in
+  let ans = List.rev ans in
+  List.hd ans, List.rev (List.tl ans)
 
 let re_file = Str.regexp "^file: \\(.*\\)$"
 let re_metadata = Str.regexp "^\\([^:]+\\): \\(.*\\)$"
@@ -95,70 +92,72 @@ let valid_metadata = ["artist"; "title"; "album"; "genre"; "date"; "track"]
 let search read write field v =
   let l =
     let ret, l = cmd read write "search" [field; v] in
-      assert (ret = "OK");
-      l
+    assert (ret = "OK");
+    l
   in
   let ans = ref [] in
   let file = ref "" in
   let metadata = ref [] in
   let add () =
     if !file <> "" then
-      ans := (Request.indicator
-                ~metadata:(Utils.hashtbl_of_list !metadata)
-                !file) :: !ans
+      ans :=
+        (Request.indicator
+           ~metadata:(Utils.hashtbl_of_list !metadata)
+           !file) :: !ans
   in
-    List.iter
-      (fun s ->
-         if Str.string_match re_file s 0 then
-           (
-             let f = Str.matched_group 1 s in
-             let prefix = conf_path_prefix#get in
-             let f = prefix ^ "/" ^ f in
-               if debug then Printf.printf "Found: %s\n%!" f;
-               add ();
-               file := f
-           )
-         else if Str.string_match re_metadata s 0 then
-           let field = Str.matched_group 1 s in
-           let field = String.lowercase field in
-           let value = Str.matched_group 2 s in
-             if List.mem field valid_metadata then
-               metadata := (field, value) :: !metadata
-      ) l;
-    add ();
-    if conf_randomize#get then
-      (
-        let ans = Array.of_list !ans in
-          Utils.randomize ans;
-          Array.to_list ans
-      )
-    else
-      List.rev !ans
+  List.iter
+    (fun s ->
+      if Str.string_match re_file s 0 then
+        let f = Str.matched_group 1 s in
+        let prefix = conf_path_prefix#get in
+        let f = prefix ^ "/" ^ f in
+        if conf_debug#get then Printf.printf "Found: %s\n%!" f;
+        add ();
+        file := f
+      else if Str.string_match re_metadata s 0 then
+        let field = Str.matched_group 1 s in
+        let field = Utils.StringCompat.lowercase_ascii field in
+        let value = Str.matched_group 2 s in
+        if List.mem field valid_metadata then
+          metadata := (field, value) :: !metadata
+    ) l;
+  add ();
+  if conf_randomize#get then
+    let ans = Array.of_list !ans in
+    Utils.randomize ans;
+    Array.to_list ans
+  else
+    List.rev !ans
 
 let re_request = Str.regexp "^\\([^=]+\\)=\\(.*\\)$"
 let re_version = Str.regexp "OK MPD \\([0-9\\.]+\\)"
 
-let mpd s ~log maxtime =
+let mpd s ~log _ =
   if not (Str.string_match re_request s 0) then
     raise (Error "Invalid request");
   let field = Str.matched_group 1 s in
   let value = Str.matched_group 2 s in
-  let socket, read, write = connect () in
+  let value =
+    let len = String.length value in
+    if len > 0 && value.[0] = '"' && value.[len-1] = '"' then
+      String.sub value 1 (len-2)
+    else
+      value
+  in
+  let _, read, write = connect () in
   let search = search read write in
   let version =
     let v = read () in
-      if Str.string_match re_version v 0 then
-        Str.matched_group 1 v
-      else
-        raise (Error "Not an MPD server")
+    if Str.string_match re_version v 0 then
+      Str.matched_group 1 v
+    else
+      raise (Error "Not an MPD server")
   in
   log (Printf.sprintf "Connected to MPD version %s\n" version);
   let files = search field value in
-    write "close\n";
-    files
+  write "close\n";
+  files
 
 let () =
-  Request.protocols#register "mpd"
-    ~sdoc:("[mpd:tag=\"value\"] finds all files with a tag equal " ^
-           "to a given value using mpd.")
-    { Request.resolve = mpd ; Request.static = false }
+  Lang.add_protocol ~doc:"Finds all files with a tag equal to a given value using mpd."
+    ~syntax:"mpd:tag=value" ~static:false "mpd" mpd
