@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -102,6 +102,6 @@ let () =
         with
           | e ->
               Main.log#f 2 "Error while running service: %s"
-                             (Utils.error_message e)
+                             (Printexc.to_string e)
        end
     | `None -> main ()
