@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -93,7 +93,7 @@ object (self)
                 Pcm.set_format dev params Pcm.Format_s16_le ;
                 read_fun <-
                   (fun pcm buf ofs len ->
-                     let sbuf = String.create (2 * 2 * len) in
+                     let sbuf = Bytes.create (2 * 2 * len) in
                      let r = Pcm.readi pcm sbuf 0 len in
                      Audio.S16LE.to_audio sbuf 0 buf ofs r;
                      r)
