@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,16 +23,16 @@
 (** A [plug] is something where plug-ins plug.
   We build [plug] on the top of [Doc.item]. *)
 
-class ['a] plug name ?(register_hook=fun _ -> ()) doc insensitive 
+class ['a] plug ?(register_hook=fun _ -> ()) doc insensitive 
                      duplicates =
-object (self)
+object
   inherit Doc.item doc
 
   val mutable plugins : (string*'a) list = []
   val mutable aliases : (string*'a) list = []
 
   method register plugin ?plugin_aliases ?doc ?sdoc v =
-    let plugin = if insensitive then String.uppercase plugin else plugin in
+    let plugin = if insensitive then Utils.StringCompat.uppercase_ascii plugin else plugin in
     let doc = match doc,sdoc with
       | (Some d), _ -> d
       | _, None -> Doc.trivial "(no doc)"
@@ -53,7 +53,7 @@ object (self)
         | None       -> ()
 
   method is_registered a = List.mem_assoc a plugins
-  method keys = List.fold_left (fun l (k,v) -> k::l) [] plugins
+  method keys = List.fold_left (fun l (k,_) -> k::l) [] plugins
   method iter ?(rev=false) f = 
     let plugins = 
       if rev then
@@ -64,7 +64,7 @@ object (self)
     List.iter (fun (k,v) -> f k v) plugins
   method get_all = plugins
   method get plugin =
-    let plugin = if insensitive then String.uppercase plugin else plugin in
+    let plugin = if insensitive then Utils.StringCompat.uppercase_ascii plugin else plugin in
       try
         Some (List.assoc plugin plugins)
       with
@@ -85,7 +85,7 @@ let plugs = new Doc.item "All the plugs"
 let create ?(duplicates=true) ?register_hook ?insensitive ?doc plugname =
   let insensitive = match insensitive with Some true -> true | _ -> false in
   let doc = match doc with None -> "(no doc)" | Some d -> d in
-  let plug = new plug ?register_hook plugname doc insensitive duplicates in
+  let plug = new plug ?register_hook doc insensitive duplicates in
     plugs#add_subsection plugname (plug:>Doc.item) ;
     plug
 
