@@ -2,7 +2,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
  *****************************************************************************)
 open Dtools
   
-let log = Dtools.Log.make [ "server" ]
-  
 let conf =
   Conf.void ~p: (Configure.conf#plug "server") "Server configuration"
     ~comments:
@@ -84,7 +82,7 @@ let conf_telnet_port =
     "Port on which the telnet server should listen"
   
 let conf_telnet_revdns =
-  Conf.bool ~p: (conf_telnet#plug "reverse_dns") ~d: true
+  Conf.bool ~p: (conf_telnet#plug "revdns") ~d: false
     "Perform reverse DNS lookup to get the client's hostname from its IP."
   
 let log = Log.make [ "server" ]
@@ -116,7 +114,7 @@ let register ns kind =
   
 let to_string = String.concat "."
   
-let rec prefix_ns cmd ns = to_string (ns @ [ cmd ])
+let prefix_ns cmd ns = to_string (ns @ [ cmd ])
   
 (* Then add your commands to that namespace *)
 let add ~ns ?usage ~descr cmd handler =
@@ -168,17 +166,15 @@ let usage () =
   let l = List.sort compare l
   in
     List.fold_left
-      (fun s (k, (h, u, _)) -> s ^ (Printf.sprintf "\r\n| %s" u)) "" l
+      (fun s (_, (_, u, _)) -> s ^ (Printf.sprintf "\r\n| %s" u)) "" l
   
 (** {1 Handling of a client} *)
 (** The very-builtin commands *)
 let () =
   let add = add ~ns: []
   in
-    (add "exit" ~descr: "Close current server session."
-       (fun args -> raise Exit);
-     add "quit" ~descr: "Close current server session."
-       (fun args -> raise Exit);
+    (add "exit" ~descr: "Close current server session." (fun _ -> raise Exit);
+     add "quit" ~descr: "Close current server session." (fun _ -> raise Exit);
      add "help" ~usage: "help [<command>]"
        ~descr: "Get information on available commands."
        (fun args ->
@@ -225,7 +221,7 @@ let exec s =
     | Not_found ->
         "ERROR: unknown command, type \"help\" to get a \
                       list of commands."
-    | e -> Printf.sprintf "ERROR: %s" (Utils.error_message e)
+    | e -> Printf.sprintf "ERROR: %s" (Printexc.to_string e)
   
 let handle_client socket ip =
   let on_error e =
@@ -237,8 +233,8 @@ let handle_client socket ip =
                       client %s."
             ip
       | Duppy.Io.Unix (c, p, m) ->
-          log#f 4 "%s" (Utils.error_message (Unix.Unix_error (c, p, m)))
-      | Duppy.Io.Unknown e -> log#f 3 "%s" (Utils.error_message e));
+          log#f 4 "%s" (Printexc.to_string (Unix.Unix_error (c, p, m)))
+      | Duppy.Io.Unknown e -> log#f 3 "%s" (Printexc.to_string e));
      Duppy e) in
   let h =
     {
@@ -310,11 +306,11 @@ let start_socket () =
   let rec incoming _ =
     ((try
         let (socket, caller) = Unix.accept sock in
-        let ip = Utils.name_of_sockaddr caller
+        let ip =
+          Utils.name_of_sockaddr ~rev_dns: conf_telnet_revdns#get caller
         in (log#f 3 "New client %s." ip; handle_client socket ip)
       with
-      | e ->
-          log#f 2 "Failed to accept new client: %S" (Utils.error_message e));
+      | e -> log#f 2 "Failed to accept new client: %S" (Printexc.to_string e));
      [ {
          Duppy.Task.priority = Tutils.Non_blocking;
          events = [ `Read sock ];
@@ -332,7 +328,7 @@ let start_socket () =
      Unix.bind sock bind_addr;
      Unix.listen sock max_conn;
      ignore
-       (Dtools.Init.at_stop
+       (Shutdown.at_stop
           (fun () ->
              (log#f 3 "Unlink %s" socket_name; Unix.unlink socket_path)));
      Unix.chmod socket_path rights;
@@ -348,19 +344,25 @@ let start_telnet () =
   let bind_addr_inet = Unix.inet_addr_of_string conf_telnet_bind_addr#get in
   let bind_addr = Unix.ADDR_INET (bind_addr_inet, port) in
   let max_conn = 10 in
-  let sock = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0
+  let sock = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+  let () =
+    (* The socket has to be closed for restart to work, and this has to be
+       done after duppy has stopped using it. *)
+    Shutdown.at_stop ~name: "Server shutdown"
+      ~depends: [ Shutdown.duppy_scheduler () ]
+      (fun () -> (log#f 3 "Closing socket."; Unix.close sock))
   in
     (* Set TCP_NODELAY on the socket *)
     (Unix.setsockopt sock Unix.TCP_NODELAY true;
      let rec incoming _ =
        ((try
            let (socket, caller) = Unix.accept sock in
-           let ip = Utils.name_of_sockaddr caller
+           let ip =
+             Utils.name_of_sockaddr ~rev_dns: conf_telnet_revdns#get caller
            in (log#f 3 "New client: %s." ip; handle_client socket ip)
          with
          | e ->
-             log#f 2 "Failed to accept new client: %S"
-               (Utils.error_message e));
+             log#f 2 "Failed to accept new client: %S" (Printexc.to_string e));
         [ {
             Duppy.Task.priority = Tutils.Non_blocking;
             events = [ `Read sock ];
@@ -383,8 +385,7 @@ let start () =
   in
     if telnet || socket
     then
-      (Tutils.need_non_blocking_queue ();
-       if telnet then start_telnet () else ();
+      (if telnet then start_telnet () else ();
        if socket then start_socket () else ())
     else ()
   
