@@ -28,8 +28,13 @@ module List = struct
         assoc_nth l (n-1) t
     | _::t -> assoc_nth l n t
 
-  let rec assoc_all x l =
+  let assoc_all x l =
     may_map (fun (y,v) -> if x = y then Some v else None) l
+
+  let rec last = function
+    | [x] -> x
+    | _::l -> last l
+    | [] -> raise Not_found
 end
 
 module String = struct
@@ -47,3 +52,19 @@ module String = struct
     in
     List.rev (aux [] 0)
 end
+
+let read_retry read s buf off len =
+  let r = ref 0 in
+  let loop = ref true in
+  while !loop do
+    let n = read s buf (off + !r) (len - !r) in
+    r := !r + n;
+    loop := !r <> 0 && !r < len
+  done;
+  !r
+
+module Unix = struct
+  include Unix
+
+  let read_retry = read_retry Unix.read
+end
