@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -145,7 +145,7 @@ let create ~wait f x s =
                  log#f 3 "Thread %S terminated." s
              with e ->
                Mutex.lock lock ;
-               let backtrace = Utils.get_backtrace () in
+               let backtrace = Printexc.get_backtrace () in
                begin match e with
                  | Exit ->
                      log#f 3 "Thread %S exited." s
@@ -153,7 +153,7 @@ let create ~wait f x s =
                      log#f 1 "Thread %S failed: %s!" s e
                  | e ->
                      log#f 1 "Thread %S aborts with exception %s!"
-                              s (Utils.error_message e)
+                              s (Printexc.to_string e)
                end ;
                if e <> Exit then
                 begin
@@ -182,6 +182,22 @@ type priority =
 
 let scheduler = Duppy.create ()
 
+let started  = ref false
+
+let started_m = Mutex.create ()
+
+let has_started = mutexify started_m (fun () ->
+  !started)
+
+let () =
+  let name = "Duppy scheduler shutdown" in
+  let f () =
+    log#f 3 "Shutting down scheduler...";
+    Duppy.stop scheduler;
+    log#f 3 "Scheduler shut down."
+  in
+  Shutdown.duppy_atom := Some (Dtools.Init.at_stop ~name f)
+
 let scheduler_log n =
   if scheduler_log#get then
     let log = Log.make [n] in
@@ -198,19 +214,16 @@ let new_queue ?priorities ~name () =
          | Some priorities ->
              Duppy.queue scheduler ~log:qlog ~priorities name
      with e ->
-       log#f 2 "Queue %s crashed with exception %s" name (Utils.error_message e) ;
-       log#f 2 "%s" (Utils.get_backtrace());
-       log#f 1 "PANIC: Liquidsoap has crashed, exiting.." ;
-       log#f 1 "Please report at: savonet-users@lists.sf.net" ;
+       log#f 2 "Queue %s crashed with exception %s\n\
+                %s" name (Printexc.to_string e) (Printexc.get_backtrace());
+       log#f 1 "PANIC: Liquidsoap has crashed, exiting.,\n\
+                Please report at: savonet-users@lists.sf.net" ;
        exit 1
    in
    ignore (create ~wait:false queue () name)
 
 let create f x name = create ~wait:true f x name
 
-let start_non_blocking = ref false
-let need_non_blocking_queue () = start_non_blocking := true
-
 let () =
   (* A dtool atom to start
    * tasks *)
@@ -224,13 +237,14 @@ let () =
         let name = Printf.sprintf "fast queue #%d" i in
           new_queue ~name ~priorities:(fun x -> x = Maybe_blocking) ()
       done;
-      if !start_non_blocking then
-        for i = 1 to non_blocking_queues#get do
-          let name = Printf.sprintf "non-blocking queue #%d" i in
-          new_queue
-            ~priorities:(fun x -> x = Non_blocking)
-            ~name ()
-        done))
+      for i = 1 to non_blocking_queues#get do
+        let name = Printf.sprintf "non-blocking queue #%d" i in
+        new_queue
+          ~priorities:(fun x -> x = Non_blocking)
+          ~name ()
+      done;
+      mutexify started_m (fun () ->
+        started := true) ()))
 
 (** Replace stdout/err by a pipe, and install a Duppy task that pulls data
   * out of that pipe and logs it.
@@ -240,6 +254,7 @@ let start_forwarding () =
     let i,o = Unix.pipe () in
       Unix.dup2 o fd ;
       Unix.close o ;
+      Unix.set_close_on_exec i ;
       i
   in
   let in_stdout = reopen Unix.stdout in
@@ -261,7 +276,7 @@ let start_forwarding () =
          handler  = f }
     in
     let len = 1024 in
-    let buffer = String.create len in
+    let buffer = Bytes.create len in
     let rec f (acc:string list) _ =
       let n = Unix.read fd buffer 0 len in
       let rec split acc i =
@@ -290,7 +305,6 @@ let () =
   ignore (Dtools.Init.at_start (fun () ->
     if Dtools.Init.conf_daemon#get then begin
       Dtools.Log.conf_stdout#set false ;
-      need_non_blocking_queue () ;
       start_forwarding ()
     end))
 
@@ -299,42 +313,53 @@ let wait c m f =
   mutexify m (fun () ->
     while not (f ()) do Condition.wait c m done) ()
 
-exception Timeout
+exception Timeout of float
 
 let error_translator =
   function
-    | Timeout ->
-        Some "Timeout while waiting on socket"
+    | Timeout f ->
+        Some (Printf.sprintf "Timed out after waiting for %.02f sec." f)
     | _ ->
         None
 
 let () = Utils.register_error_translator error_translator
 
-(* Wait for [`Read], [`Write] or [`Both] for at most
- * [timeout] seconds on the given [socket]. Raises [Timeout]
+type event = [
+  | `Read of Unix.file_descr
+  | `Write of Unix.file_descr
+  | `Both of Unix.file_descr
+]
+
+(* Wait for [`Read socker], [`Write socket] or [`Both socket] for at most
+ * [timeout] seconds on the given [socket]. Raises [Timeout elapsed_time]
  * if timeout is reached. *)
-let wait_for ?(log=fun _ -> ()) event socket timeout =
-  let max_time = Unix.gettimeofday () +. timeout in
-  let r, w = 
+let wait_for ?(log=fun _ -> ()) event timeout =
+  let start_time =
+    Unix.gettimeofday ()
+  in
+  let max_time =
+    start_time +. timeout
+  in
+  let r, w =
     match event with
-      | `Read -> [socket],[]
-      | `Write -> [],[socket]
-      | `Both -> [socket],[socket]
+      | `Read socket -> [socket],[]
+      | `Write socket -> [],[socket]
+      | `Both socket -> [socket],[socket]
   in
   let rec wait t =
     let l,l',_ = Unix.select r w [] t in
     if l=[] && l'=[] then begin
-      log (Printf.sprintf "No network activity for %.02f second(s)." t);
       let current_time = Unix.gettimeofday () in
       if current_time >= max_time then
        begin
-        log "Network activity timeout!" ;
-        raise Timeout 
+        log "Timeout reached!" ;
+        raise (Timeout (current_time -. start_time))
        end
       else
         wait (min 1. (max_time -. current_time))
     end
-  in wait (min 1. timeout)
+  in
+  wait (min 1. timeout)
 
 (** Wait for some thread to crash *)
 let run = ref true
