@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
   raises an exception. *)
 val create : ('a -> unit) -> 'a -> string -> Thread.t
 val main : unit -> unit
+val has_started : unit -> bool
 val shutdown : unit -> unit
 
 (** Special exception allowed for "clean" termination of Tutils threads.
@@ -55,10 +56,6 @@ type priority =
 (** task scheduler *)
 val scheduler : priority Duppy.scheduler
 
-(** Ask for a special queue that treats exclusively non blocking tasks.
-  * This is need for things such as the server or harbor to run smoothly. *)
-val need_non_blocking_queue : unit -> unit
-
 (** {1 Misc} *)
 
 (** Waits for [f()] to become true on condition [c].
@@ -69,12 +66,17 @@ val wait : Condition.t -> Mutex.t -> (unit -> bool) -> unit
 (** Make a function work in critical section, protected by a given lock. *) 
 val mutexify : Mutex.t -> ('a -> 'b) -> ('a -> 'b)
 
-exception Timeout
+exception Timeout of float
 
-(* Wait for [`Read], [`Write] or [`Both] for at most
- * [timeout] seconds on the given [socket]. Raises [Timeout]
- * if timeout is reached. *)
-val wait_for : ?log:(string -> unit) -> [`Read|`Write|`Both] -> Unix.file_descr -> float -> unit
+type event = [
+  | `Read of Unix.file_descr
+  | `Write of Unix.file_descr
+  | `Both of Unix.file_descr
+]
+
+(* Wait some events: [`Read socket], [`Write socket] or [`Both timeout]
+ * Raises [Timeout elapsed_time] if timeout is reached. *)
+val wait_for : ?log:(string -> unit) -> event -> float -> unit
 
 (** [finalize ~k f] calls [f] and returns it result,
   * and always executes [k], even when [f] raises an exception. *)
@@ -86,7 +88,7 @@ val finalize : k:(unit -> unit) -> (unit -> 'a) -> 'a
   * This is meant to be used for assertions. *)
 val seems_locked : Mutex.t -> bool
 
-(** Thread-safe equivalent to Lazy.lazy_from_fun. *)
+(** Thread-safe equivalent to Lazy.from_fun. *)
 val lazy_cell : (unit -> 'a) -> (unit -> 'a)
 
 (** Preemptive stoppable thread.
