@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2013 Savonet team
+  Copyright 2003-2017 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+let pi = 4. *. atan 1.
+
 let get_some = function
   | Some x -> x
   | None -> assert false
@@ -96,7 +98,59 @@ let hashtbl_get : ('a,'b) Hashtbl.t -> 'a -> 'b option =
   fun h k ->
     try Some (Hashtbl.find h k) with Not_found -> None
 
-(** Unescapt a string. *)
+(* Backward-compatible API.. *)
+
+module LazyCompat =
+struct
+  let from_fun f = lazy (f ())
+  let from_val v = lazy v
+  (* Make compiler happy.. *)
+  let () =
+    ignore(from_fun (fun () -> ()));
+    ignore(from_val ())
+  include Lazy
+end
+
+exception Not_implemented
+
+module type StringWrapper =
+sig
+  include module type of String
+  val capitalize : string -> string
+  val uncapitalize : string -> string
+  val lowercase : string -> string
+  val uppercase : string -> string
+end
+
+module StringWrapper : StringWrapper =
+struct
+  let capitalize _ = raise Not_implemented
+  let uncapitalize _ = raise Not_implemented
+  let lowercase _ = raise Not_implemented
+  let uppercase _ = raise Not_implemented
+  let () =
+    let e f = try ignore(f "") with Not_implemented -> () in
+    e capitalize; e uncapitalize;
+    e lowercase; e uppercase
+  include String
+end
+
+module StringCompat =
+struct
+  let capitalize_ascii = StringWrapper.capitalize
+  let uncapitalize_ascii = StringWrapper.uncapitalize
+  let lowercase_ascii = StringWrapper.lowercase
+  let uppercase_ascii = StringWrapper.uppercase
+
+  let () =
+    let e f = try ignore(f "") with Not_implemented -> () in
+    e capitalize_ascii; e uncapitalize_ascii;
+    e lowercase_ascii; e uppercase_ascii
+
+  include String
+end
+
+(** Unescape a string. *)
 let unescape s =
   try
     Scanf.sscanf s "%S" (fun u -> u)
@@ -110,9 +164,9 @@ let unescape_char c s =
     if pos >= len then
       (if escaped then Printf.sprintf "%s\\" cur else cur)
     else
-      let c   = s.[pos] in
+      let d   = s.[pos] in
       let pos = pos + 1 in
-      match c with
+      match d with
         | '\\' when not escaped ->
             f ~escaped:true cur pos
         | x    when x = c ->
@@ -173,15 +227,6 @@ let rec may_map f = function
       end
   | [] -> []
 
-let really_read fd buf ofs len =
-  let l = ref 0 in
-  let r = ref (-1) in
-    while !l < len && !r <> 0 do
-      r := Unix.read fd buf !l (len - !l);
-      l := !l + !r
-    done;
-    !l
-
 (* Read all data from a given filename.
  * We cannot use really_input with the 
  * reported length of the file because
@@ -193,7 +238,7 @@ let really_read fd buf ofs len =
 let read_all filename =
   let channel = open_in filename in
   let buflen = 1024 in
-  let tmp = String.create 1024 in
+  let tmp = Bytes.create 1024 in
   let contents = Buffer.create buflen in
   let rec read () =
     let ret = input channel tmp 0 1024 in
@@ -239,40 +284,7 @@ let exception_printer e =
     (fun cur f -> if cur <> None then cur else f e)
     None error_translators
 
-let register_printer _ = 
-    raise Not_found
-
-(* Exception backtrace printing.
- * This is used in Threads where 
- * the backtrace seems to be lost
- * otherwise. *)
-
-(* Default implementation when Printexc does
- * not implement it. *)
-let get_backtrace () = 
-  "Liquidsoap not compiled with ocaml >= 3.11, \
-   cannot print stack backtrace"
-
-(* Open Printexc, which overrides register_printer 
- * and get_backtrace. *)
-open Printexc
-
-let get_backtrace = get_backtrace
-
-let printexc_has_register = 
-  try
-    register_printer exception_printer ;
-    true
-  with
-    | Not_found -> false
-
-let error_message e = 
-  if printexc_has_register then
-    Printexc.to_string e
-  else
-    match exception_printer e with
-      | Some s -> s
-      | None   -> Printexc.to_string e
+let () = Printexc.register_printer exception_printer
 
 (** Perfect Fisher-Yates shuffle
   * (http://www.nist.gov/dads/HTML/fisherYatesShuffle.html). *)
@@ -425,14 +437,23 @@ let interpolate =
         interpolate (process_if s)
 
 (** [which s] is equivalent to /usr/bin/which s, raises Not_found on error *)
-let which =
-  let path =
-    let s = Sys.getenv "PATH" in
-      Str.split (Str.regexp_string ":") s
+let which ~path s =
+  let test fname =
+    try
+      Unix.access fname [Unix.X_OK];
+      true
+    with _ -> false
   in
-    fun s ->
-      if Sys.file_exists s then s else
-        List.find Sys.file_exists (List.map (fun d -> d^"/"^s) path)
+  if test s then s else
+    List.find test (List.map (fun d -> Filename.concat d s) path)
+
+(** Get current timezone. *)
+external timezone : unit -> int = "liquidsoap_get_timezone"
+
+let string_of_timezone tz =
+  (* TODO: not sure about why we need this... *)
+  let tz = -tz in
+  Printf.sprintf "%+03d%02d" (tz/3600) ((abs (tz/60)) mod 60)
 
 (** Very partial strftime clone *)
 let strftime str : string =
@@ -445,6 +466,7 @@ let strftime str : string =
       "m", Printf.sprintf "%02d" (t.Unix.tm_mon + 1) ;
       "Y", string_of_int (t.Unix.tm_year + 1900)     ;
       "w", string_of_int (t.Unix.tm_wday)            ;
+      "z", string_of_timezone (timezone ())          ;
       "%", "%" ]
   in
   let subst sub =
@@ -531,8 +553,8 @@ let decode64 s =
         | _ -> failwith "decode64: invalid encoding"
     in
     let len = List.length result in
-    let s = String.make len ' ' in
-      ignore (List.fold_left (fun i c -> s.[i] <- c ; i-1) (len-1) result) ;
+    let s = Bytes.make len ' ' in
+      ignore (List.fold_left (fun i c -> Bytes.set s i c ; i-1) (len-1) result) ;
       s
 
 (** Base 64 encoding. *)
@@ -543,9 +565,9 @@ let encode64 s =
   let extra = String.length s mod 3 in
   let s = match extra with 1 -> s ^ "\000\000" | 2 -> s ^ "\000" | _ -> s in
   let n = String.length s in
-  let dst = String.create (4 * (n/3)) in
+  let dst = Bytes.create (4 * (n/3)) in
     for i = 0 to n/3 - 1 do
-      let (:=) j v = dst.[i*4+j] <- digit.[v] in
+      let (:=) j v = Bytes.set dst (i*4+j) digit.[v] in
       let c j = int_of_char s.[i*3+j] in
       let c0 = c 0 and c1 = c 1 and c2 = c 2 in
         0 := c0 lsr 2 ;
@@ -554,10 +576,10 @@ let encode64 s =
         3 := c2 land 63
     done ;
     if extra = 1 then begin
-      dst.[4*(n/3)-2] <- '=' ;
-      dst.[4*(n/3)-1] <- '='
+      Bytes.set dst (4*(n/3)-2) '=' ;
+      Bytes.set dst (4*(n/3)-1) '='
     end else if extra = 2 then
-      dst.[4*(n/3)-1] <- '=' ;
+      Bytes.set dst (4*(n/3)-1) '=' ;
     dst
 
 (** Get a file/uri extension. *)
@@ -568,11 +590,11 @@ let get_ext s =
  try
   let rex = Pcre.regexp "\\.([a-zA-Z0-9]+)[^.]*$" in
   let ret = Pcre.exec ~rex s in
-  String.lowercase (Pcre.get_substring ret 1)
+  StringCompat.lowercase_ascii (Pcre.get_substring ret 1)
  with
    | _ -> raise Not_found
 
-let name_of_sockaddr ?(rev_dns=true) ?(show_port=false) a =
+let name_of_sockaddr ?(rev_dns=false) ?(show_port=false) a =
   match a with
     | Unix.ADDR_UNIX s -> Printf.sprintf "unix socket %S" s
     | Unix.ADDR_INET (x,p) ->
@@ -604,7 +626,7 @@ let normalize_parameter_string s =
   let s = Pcre.substitute ~pat:" +$" ~subst:(fun _ -> "") s in
   let s = Pcre.substitute ~pat:"( +|/+|-+)" ~subst:(fun _ -> "_") s in
   let s = Pcre.substitute ~pat:"\"" ~subst:(fun _ -> "") s in
-  let s = String.lowercase s in
+  let s = StringCompat.lowercase_ascii s in
   (* Identifiers cannot begin with a digit. *)
   let s = if Pcre.pmatch ~pat:"^[0-9]" s then "_"^s else s in
   s
@@ -631,3 +653,93 @@ let reopen_in inchan filename =
   in
   Unix.dup2 fd2 fd1;
   Unix.close fd2
+
+(* See: http://www.onicos.com/staff/iz/formats/ieee.c *)
+let float_of_extended_float bytes =
+  let float_of_unsigned u =
+    let ( - ) = Int32.sub in
+    let f = (Int32.shift_left Int32.one 31) - Int32.one in
+    (Int32.to_float (u - f - Int32.one)) +. 2147483648.
+  in
+  let expon = (((int_of_char bytes.[0]) land 0x7F) lsl 8) lor ((int_of_char bytes.[1]) land 0xff) in
+  let boc c = Int32.of_int (int_of_char c land 0xff) in
+  let ( lsl ) = Int32.shift_left in
+  let ( lor ) = Int32.logor in
+  let hiMant =
+    ((boc bytes.[2]) lsl 24) lor
+      ((boc bytes.[3]) lsl 16) lor
+      ((boc bytes.[4]) lsl 8) lor
+      (boc bytes.[5])
+  in
+  let loMant =
+    ((boc bytes.[6]) lsl 24) lor
+      ((boc bytes.[7]) lsl 16) lor
+      ((boc bytes.[8]) lsl 8) lor
+      (boc bytes.[9])
+  in
+  if expon = 0 && hiMant = Int32.zero && loMant = Int32.zero then
+    0.
+  else
+    begin
+      if expon = 0x7fff then
+        nan
+      else
+        begin
+          let expon = expon - 16383 - 31 in
+          let f = ldexp (float_of_unsigned hiMant) expon in
+          let f = f +. ldexp (float_of_unsigned loMant) (expon-32) in
+          if (int_of_char bytes.[0]) land 0x80 <> 0 then
+            -1. *. f
+          else
+            f
+        end
+    end
+
+(* From OCaml *)
+let file_extension_len ~dir_sep name =
+  let rec check i0 i =
+    if i < 0 || name.[i] = dir_sep then 0
+    else if name.[i] = '.' then check i0 (i - 1)
+    else String.length name - i0
+  in
+  let rec search_dot i =
+    if i < 0 || name.[i] = dir_sep then 0
+    else if name.[i] = '.' then check i (i - 1)
+    else search_dot (i - 1)
+  in
+  search_dot (String.length name - 1)
+
+let file_extension ?(dir_sep=Filename.dir_sep) name =
+  let dir_sep = dir_sep.[0] in
+  let l = file_extension_len ~dir_sep name in
+  if l = 0 then "" else String.sub name (String.length name - l) l
+
+let quote s =
+  let quote = '"' in
+  let quotequote = "\\\"" in
+  let escp = '\\' in
+  let quoteescp = "\\\\" in
+  let l = String.length s in
+  let b = Buffer.create (l + 20) in
+  Buffer.add_char b quote;
+  for i = 0 to l - 1 do
+    match s.[i] with
+      | c when c = quote -> Buffer.add_string b quotequote
+      | c when c = escp -> Buffer.add_string b quoteescp
+      | c -> Buffer.add_char b c
+  done;
+  Buffer.add_char b quote;
+  Buffer.contents b
+
+let environment () =
+  let l = Unix.environment () in
+  (* Split at first occurence of '='. Return v,"" if
+   * no '=' could be found. *)
+  let split s =
+    try
+      let pos = String.index s '=' in
+      String.sub s 0 pos, String.sub s (pos+1) (String.length s - pos - 1)
+    with _ -> s,""
+  in
+  let l = Array.to_list l in
+  List.map split l
